      real*8 function sig_tgvdvcs(Elab,E_prime,thetaEscat,
     1   Ep,Pp,thetag,phi,eps,qsq,tt,W_gev,xB,first)

c Note, all units have been passed in GeV.

c-------------------------- TgenDVCS.cxx TGVKelly.cxx --------------------------*
c                                                                               |
c Author : C. Munoz Camacho and H. Moutarde (CEA-Saclay, IRFU/SPhN).            |
c v1.4, July, 25th 2008.                                                        |
c                                                                               |
c Computation of the differential cross sections of the Bethe-Heitler and Deeply|
c Virtual Compton Scattering processes (and also of the interference of the two |
c processes).                                                                   |
c                                                                               |
c The formulas are derived from a Mathematica package written by P. Guichon     |
c (CEA-Saclay, IRFU/SPhN) and M. Vanderhaegen (W&M College), see arXiv:0808.????|
c                                                                               | 
c-------------------------------------------------------------------------------*
c adapted for SIMC by G.M. Huber, 20.02.13

      implicit none
      include 'constants.inc'
      include 'simulate.inc'

      integer TargetPolar,BeamHeli,BeamCharge,opt
      data TargetPolar/0/
      data BeamCharge/-1/
      data opt/0/   ! opt=1 returns BH cross section only
 
c--------------------------- Kinematic Variables -------------------------------*
c All quantities are in GeV or GeV^2

c Frame dependent scalars 
     
      real*8 ELab     ! Beam energy
      real*8 E_prime,thetaEscat ! scattered electron energy and angle (LAB)
      real*8 thetag,thetacm ! CM Angle between real and virtual photons
      real*8 phi
      real*8 Ep,Pp    ! Lab proton energy and momentum
      real*8 qpPerp   ! Component (here x-axis) of the real photon 3-momentum 
                      ! orthogonal to the virtual photon trajectory (here z-axis) 
                      ! in the hadronic plane (here xz-plane)
      real*8 Omega    ! (Function of) the linear polarization rate
      real*8 y        ! Variable used in the computation of Omega
      real*8 e
      data e /0.30282211985966434/
    
c Invariant scalars
         
      real*8 xB	      ! Bjorken variable
      real*8 xBMin, xBMax ! Boundaries on the xB physical region
      real*8 qsq      ! Virtuality of the photon
      real*8 eps2
      real*8 tt       ! Mandelstam variable (square of the 4-momentum transfer)
      real*8 tmin
      real*8 s,s_test ! Mandelstam variable (square of the total incoming 4-momentum)
      real*8 Q        ! Photon virtual mass i.e. square root of Q2   
      real*8 PhaseSpace ! Differential element of cross section
      real*8 m_p      ! Proton mass

c 4-vectors defined in the CM frame
c index: 1=x, 2=y, 3=z, 4=time
    
      real*8 qCM(4)  ! Virtual photon (propagates along z-axis)
      real*8 pCM(4)  ! Incoming proton (propagates along z-axis)
      real*8 qpCM(4) ! Real photon (defines hadronic plane xz)
      real*8 ppCM(4) ! Outgoing proton

      data qCM/0.,0.,0.,0./
      data pCM/0.,0.,0.,0./
      data qpCM/0.,0.,0.,0./
      data ppCM/0.,0.,0.,0./

c--------------------------- Compton Form Factors ------------------------------*

      logical first,InitMvcs
      integer i,j,k,l,iQ2,iXb,it,iGPD
      integer Q2_0,Q2_1,xB_0,xB_1,t_0,t_1

      real*8 V(8,13,80,90) ! index: iGPD=1,8, iQ2=1,13, iXb=1,80, it=1,90
      real*8 CFF(8)        ! interpolated CFF at Q2,Xb,t of event
      real*8 ReH,ImH,ReE,ImE,ReHT,ImHT,ReET,ImET
      equivalence (CFF(1),ReH)
      equivalence (CFF(2),ImH)
      equivalence (CFF(3),ReE)
      equivalence (CFF(4),ImE)
      equivalence (CFF(5),ReHT)
      equivalence (CFF(6),ImHT)
      equivalence (CFF(7),ReET)
      equivalence (CFF(8),ImET)

      real*8 Q2d,xBd,td
      real*8 c00,c10,c01,c11,c0,c1

c--------------------- (Combinations of) helicity amplitudes -------------------*

      real*8 Jem(4,3)   ! Helicity amplitudes of the interference process 
                        ! assuming the real photon has helicity +1.
      real*8 RMvcs(4,3) ! Real part of the helicity amplitudes of the VCS 
                        ! process assuming the real photon has helicity +1.
      real*8 IMvcs(4,3) ! Imaginary part of the helicity amplitudes of the 
                        ! VCS process assuming the real photon has helicity +1.
     
c--------------- Expansion of cross sections for harmonic analysis -------------*

c Bethe Heitler process
     
      real*8 SigmaBHPol0(4)  ! coefficients for the unpolarized cross section
      real*8 SigmaBHPolX(2)  ! coefficients for the x-polarized cross section 
      real*8 SigmaBHPolY     ! coefficient for the y-polarized cross section
      real*8 SigmaBHPolZ(2)  ! coefficients for the z-polarized cross section

      real*8 F1              ! Dirac form factor
      real*8 F2              ! Pauli form factor
      real*8 Ge, Gm          ! Sachs parametrization

      real*8 DDDC            ! denominator of BH cross section
      real*8 SigmaBHp,SigmaBHm 
      real*8 XSecBHp,XSecBHm ! BH cross sect for +,- beam helicities   
     
c Virtual Compton Scattering process
     
      real*8 SigmaVCSPol0(5) ! coefficients for the unpolarized cross section
      real*8 SigmaVCSPolX(4) ! coefficients for the x-polarized cross section     
      real*8 SigmaVCSPolY(5) ! coefficients for the y-polarized cross section     
      real*8 SigmaVCSPolZ(4) ! coefficients for the z-polarized cross section

      real*8 Ur(100)  ! Coefficients of the expansion of the interference cross 
                      ! section wrt (combinations of) helicity amplitudes

      real*8 SigmaVCSp,SigmaVCSm
      real*8 XSecVCSp,XSecVCSm ! VCS cross sect for +,- beam helicities   
    
c Interference
     
      real*8 SigmaIPol0(8)   ! coefficients for the unpolarized cross section     
      real*8 SigmaIPolX(8)   ! coefficients for the x-polarized cross section     
      real*8 SigmaIPolY(8)   ! coefficients for the x-polarized cross section
      real*8 SigmaIPolZ(8)   ! coefficients for the x-polarized cross section

      real*8 SigmaIp,SigmaIm
      real*8 XSecp,XSecm     ! Interf cross sect for +,- beam helicities   

c Beam Helicity Cross Section Sum and Difference

      real*8 ConvGeV2nbarn   ! Changement d'unités
      data ConvGeV2nbarn/0.389379304e+6/

      real*8 SigmaTotPlus, SigmaTotMoins
      real*8 XSecSum,XSecDif
      real*8 sig_tgendvcs    ! d4sigma/dQ2dxdtdphi [nb/GeV]
      real*8 Jac6            ! Jacobian for d6sigma required by SIMC
      real*8 sig_tgvdvcs     ! d6sigma/dOmega_p/dE_p/dOmega_e/dE_e [ub/MeV^2/sr^2]
     
c------------------- Read in Compton Form Factors (CFF) ------------------------*
c Note, the units in this file are all in GeV, GeV^2

      if (first) then
         write(6,*)' reading CFFoutput_LO.dat'
         open(unit=31,file='CFFoutput_LO.dat',readonly,err=100)
         do iQ2=1,13
           do iXb=1,80
             do it=1,90
               read(31,*,err=100)(Q2d,xBd,td,V(i,iQ2,iXb,it), i=1,8)
             enddo
           enddo
         enddo
      endif

c---------------------------- MakeKinematics -----------------------------------*
c Global variables xB, Q2, t come from main SIMC
c Compute 4-vector components of the particles involved in CM frame 
c    virtual photon's 4-momentum qCM,
c    real photon's 4-momentum qpCM, 
c    incoming proton's 4-momentum pCM,  
c    outgoing proton's 4-momentum ppCM,   
c    thetag angle between the trajectories of real and virtual photons.  
c It also evaluates the parameter Omega and the invariant phase space PhaseSpace.

      m_p = Mp/1.e3    ! value from constants.inc
      Q   = sqrt(qsq)

      s   = W_gev**2
c     s   = TMath::Power(M,2) - TMath::Power(Q,2) + TMath::Power(Q,2)/xB;
      s_test= m_p**2 - qsq + qsq/xB
      write(6,*)' s ',s,s_test 

c Test : realistic kinematic configuration ?
c The value of xBMin comes from the requirement of omega to be real, and 
c the value of xBMax expresses the fact that s >= 0. 
          
      xBMin = 2.*ELab*qsq/(m_p*(4*(ELab**2)-qsq))
      xBMax = qsq/(qsq-(m_p**2))

      if ( xBMin < xB .or. xB < xBMax ) then
	 write(6,*)' TGVKelly : Unrealistic kinematic configuration '
         write(6,*)'  -- xB isnt in the physical region !'
         write(6,*)' xB = ',xB
         write(6,*)' xBMin = ',xBMin 
         write(6,*)' xBMax = ',xBMax
         stop
      endif

c Timelike coordinate

      qCM(4)  = ((qsq*(1-2*xB))/(2.*sqrt((m_p**2) +(qsq*(1-xB))/xB)*xB))
      qpCM(4) = (-(qsq*(-1+xB))/(2.*sqrt((m_p**2) +(qsq*(1-xB))/xB)*xB))
      pCM(4)  = ((qsq +2*xB*m_p**2)/(2.*sqrt(m_p**2+(qsq*(1-xB))/xB)*xB))
      ppCM(4) = ((2*m_p**2+(qsq*(1-xB))/xB)/(2.*sqrt(m_p**2+(qsq*(1-xB))/xB)))
     
c Spacelike coordinates
     
      qpCM(1) = (sqrt((-((m_p**2)*(tt**2)*(xB**2)) + 
     1    qsq*tt*xB*(tt*(-1 + xB) - 2*xB*(m_p**2)) + 
     2    (qsq**2)*(tt*(-1 + xB) - (m_p**2)*(xB**2)))/
     3    ((qsq**2) + 4*(m_p**2)*qsq*(xB**2))))

      qCM(3) = (sqrt((qsq**2 + 4*(m_p**2)*qsq*(xB**2))/
     1    (xB*(qsq*(1 - xB) + (m_p**2)*xB)))/2.)

      qpCM(3) = (((qsq**2)*(-1 + xB) - 2*(m_p**2)*tt*(xB**2) + 
     1    2*qsq*xB*(tt*(-1 + xB) - (m_p**2)*xB))/
     2    (2.*xB*(qsq*(-1 + xB) - (m_p**2)*xB)*
     3    sqrt(((qsq**2) + 4*(m_p**2)*qsq*(xB**2))/
     4    (xB*(qsq*(1 - xB) + (m_p**2)*xB)))))

      pCM(3)  = (-qCM(3))
      ppCM(3) = (-qpCM(3))
      qpPerp  = qpCM(1)
     
      thetacm = ACos(qpCM(3)/qpCM(4))
          
c Omega
          
      y = (-qsq + 4*ELab*m_p*xB)/sqrt(qsq**2 + 4*(m_p**2)*qsq*(xB**2))

      if ( y .lt 1. ) then
         write(6,*) ' TGVKelly : ELab ',ELab,' GeV is too small.'
         write(6,*)'  CosH[ome]= 'y 
         stop
      endif

      Omega = Log(y + sqrt(-1 + y)*sqrt(1 + y))
          
c PhaseSpace
          
      PhaseSpace = ((e**6)*qsq)/
     1    ( 4096.*(pi**5)* sqrt( 4*(m_p**2)*qsq +(qsq**2)/(xB**2) )
     2    *(xB**2)* ( qsq/(4.*xB) + 
     3    ( sqrt( 4*(m_p**2)*qsq +(qsq**2/xB**2)) *CosH(Omega)) /4.)**2)
          
      if (debug(2)) then
         write(6,*)' Q ',Q
         write(6,*)' qpPerp ',qcPerp
         write(6,*)' s ',s,s_test
         write(6,*)' t ',tt
         write(6,*)' theta* ',thetag,thetacm
         write(6,*)' xB ',xB
         write(6,*)' pCM  ',pCM(1),pCM(2),pCM(3),pCM(4)
         write(6,*)' ppCM ',ppCM(1),ppCM(2),ppCM(3),ppCM(4)
         write(6,*)' qCM  ',qCM(1),qCM(2),qCM(3),qCM(4)
         write(6,*)' qpCM ',qpCM(1),qpCM(2),qpCM(3),qpCM(4)
	endif 

c---------------- Interpolate CFF to Q2, xB, t of the event --------------------*

      eps2 = 4.*(xB**2)*(m_p**2)/qsq
      write(6,*)' eps ',eps,eps**2,eps2

      tmin = -qsq*(2.*(1.-xB)*(1-Sqrt(1+eps2))+eps2)/(4.*xB*(1.-xB)+eps2)

      if (qsq.lt.1. .or. qsq.gt.13. .or. xB.lt.0.1 .or. xB.gt.0.9 
     1    .or. tt.lt.-3.0) then
	 write(6,*)'Kinematics (Q2,xb,t,tmin) out of range for xsec evaluation'
         write(6,*)' Q2 ',qsq
         write(6,*)' xB ',xB
         write(6,*)' t  ',tt,tmin
	 stop
      endif

c add +1 to all indices, since fortran starts at 1, but c++ starts at 0.
      Q2_0 = ifix(Q2-1.)+1
      Q2_1 = ifix(Q2-1.)+1+1
      xB_0 = ifix( (xB-0.1)*79/0.8 )+1
      xb_1 = ifix( (xB-0.1)*79/0.8 )+1+1
      t_0  = ifix( -(t-0.)*89/3. )+1
      t_1  = ifix( -(t-0.)*89/3. )+1+1

      Q2d = (Q2-(1+Q2_0))/((1+Q2_1)-(1+Q2_0))
      xBd = (xB-(0.1+0.8*xB_0/79.))/
     1    ((0.1+0.8*xB_1/79.)-(0.1+0.8*xB_0/79.))
      td  = (tt-(0.-3.*t_0/89.))/((0.-3.*t_1/89.)-(0.-3.*t_0/89.))

      do iGPD=1,8
         c00=V(iGPD,Q2_0,xb_0,t_0)*(1-Q2d)+V(iGPD,Q2_1,xb_0,t_0)*Q2d
         c10=V(iGPD,Q2_0,xb_1,t_0)*(1-Q2d)+V(iGPD,Q2_1,xb_1,t_0)*Q2d
         c01=V(iGPD,Q2_0,xb_0,t_1)*(1-Q2d)+V(iGPD,Q2_1,xb_0,t_1)*Q2d
         c11=V(iGPD,Q2_0,xb_1,t_1)*(1-Q2d)+V(iGPD,Q2_1,xb_1,t_1)*Q2d
    
         c0=c00*(1-xBd)+c10*xBd
         c1=c01*(1-xBd)+c11*xBd
    
         CFF(iGPD)=c0*(1-td)+c1*td
      enddo

c------- MakeVCSHelicityAmplitudes(ReH,ImH,ReE,ImE,ReHT,ImHT,ReET,ImET) --------*
c Compute the hadronic helicity amplitudes RMvcs and IMvcs at leading twist.    |
c The formula depend on the real and imaginary part of the Compton Form Factor: |
c    ReH, ImH, ReE, ImE, ReHT, ImHT, ReET, ImET.                                |
c-------------------------------------------------------------------------------*
c need to add +1 to all indices, since fortran starts at 1, but c++ starts at 0.

      RMvcs(1,1)=0
      RMvcs(1,2)=0
      RMvcs(1,3)=-((-4*ReH*(-1+xB) -ReE*(xB**2))/(sqrt(1-xB)*(-2+xB)))
      RMvcs(2,1)=0
      RMvcs(2,2)=0
      RMvcs(2,3)=-((qpPerp*ReET*xB)/(sqrt(1-xB)*(-2*m_p + m_p*xB)))
      RMvcs(3,1)=0
      RMvcs(3,2)=0
      RMvcs(3,3)=(qpPerp*ReE)/(m_p*sqrt(1-xB))
      RMvcs(4,1)=0
      RMvcs(4,2)=0
      RMvcs(4,3)=-((-4*ReHT*(-1+xB)-ReET*(xB**2))/(sqrt(1-xB)*(-2+xB)))

      IMvcs(1,1)=0
      IMvcs(1,2)=0
      IMvcs(1,3)=-((-4*ImH*(-1+xB) -ImE*(xB**2))/(sqrt(1-xB)*(-2+xB)))
      IMvcs(2,1)=0
      IMvcs(2,2)=0
      IMvcs(2,3)=-((ImET*qpPerp*xB)/(sqrt(1-xB)*(-2*m_p + m_p*xB)))
      IMvcs(3,1)=0
      IMvcs(3,2)=0
      IMvcs(3,3)=(ImE*qpPerp)/(m_p*sqrt(1-xB))
      IMvcs(4,1)=0
      IMvcs(4,2)=0
      IMvcs(4,3)=-((-4*ImHT*(-1+xB)-ImET*(xB**2))/(sqrt(1-xB)*(-2+xB)))      
    
      if (debug(2)) then
        do i=1,4
          write(6,*)' IMvcs ',i,(IMvcs(i,j),j=1,3)
          write(6,*)' RMvcs ',i,(RMvcs(i,j),j=1,3)
	enddo
      endif

c------------------------------- MakeBHCrossSections() -------------------------*
c Computes all the stuff to evaluate the cross section assuming the hadronic    |
c helicity amplitudes are given, i.e. initializes :                             |
c   - the helicity amplitudes Jem,                                              |
c   - the SigmaBHPol's.        
c-------------------------------------------------------------------------------*

c proton form factors
      F1=(4.*(m_p**2) - 2.79285*tt) /
     1    ( ((1. - 1.4084507042253522*tt)**2)*(4.*(m_p**2)-tt) )
      F2=(7.1714*(m_p**2)) / 
     1    ( ((1 -1.4084507042253522*tt)**2)*(4*(m_p**2)-tt) )
      Gm=2.79285/( (1 - 1.4084507042253522*tt)**2 )
      Ge=1./( (1 - 1.4084507042253522*tt)**2)
     
c-------------- Helicity amplitudes of the interference process ----------------*
c need to add +1 to all indices, since fortran starts at 1, but c++ starts at 0.

c iexact=1:
c No approximations apart those in the computation of the hadronic helicity 
c amplitudes in terms of the Compton Form Factors.

      if (iexact.eq.1) then

        Jem(1,1)=(-2*sqrt(2.)*qCM(3)*qpCM(1)*F2*
     1   (sqrt(pCM(4) - m_p)*sqrt(ppCM(4) - m_p) - 
     2   sqrt(pCM(4) +m_p)*sqrt(ppCM(4) +m_p))*sqrt(s)*cos(thetag/2.))/
     3   (m_p*sqrt((qsq**2) - 4*s*tt)) - 
     4   (2*sqrt(2.)*(qCM(3) + qpCM(4))*Gm*
     5   (sqrt(ppCM(4) - m_p)*sqrt(pCM(4) + m_p) + 
     6   sqrt(pCM(4) - m_p)*sqrt(ppCM(4) + m_p))*sqrt(s)*sin(thetag/2.))/
     7   sqrt((qsq**2) - 4*s*tt)

        Jem(1,2)=-((F2*(sqrt(pCM(4) - m_p)*sqrt(ppCM(4) - m_p) - 
     1   sqrt(pCM(4) + m_p)*sqrt(ppCM(4) + m_p))*
     2   (2*(m_p**2) + qsq + 2*s)*sqrt(-(s*tt))*cos(thetag/2.))/
     3   (m_p*sqrt(s)*sqrt((qsq**2) - 4*s*tt))) - 
     4   (8*Gm*(sqrt(ppCM(4) - m_p)*sqrt(pCM(4) + m_p) + 
     5   sqrt(pCM(4) - m_p)*sqrt(ppCM(4) + m_p))*s*sqrt(-tt)*
     6   ((qCM(3) - qpCM(3))*cos(thetag/2.) - 
     7   qpCM(1)*sin(thetag/2.)))/(qsq*sqrt((qsq**2) - 4*s*t))

        Jem(1,3)=(2*sqrt(2.)*qCM(3)*qpCM(1)*F2*
     1   (sqrt(pCM(4) - m_p)*sqrt(ppCM(4) - m_p) - 
     2   sqrt(pCM(4) + m_p)*sqrt(ppCM(4) +m_p))*sqrt(s)*cos(thetag/2.))/
     3   (m_p*sqrt((qsq**2) - 4*s*tt)) + 
     4   (2*sqrt(2.)*(qCM(3) + qpCM(4))*Gm*
     5   (sqrt(ppCM(4) - m_p)*sqrt(pCM(4) + m_p) + 
     6   sqrt(pCM(4) - m_p)*sqrt(ppCM(4) + m_p))*sqrt(s)*sin(thetag/2.))/
     7   sqrt((qsq**2) - 4*s*tt)

        Jem(2,1)=sqrt(2.)*Gm*(-(sqrt(ppCM(4) - m_p)*sqrt(pCM(4) +m_p)) + 
     1   sqrt(pCM(4) - m_p)*sqrt(ppCM(4) + m_p))*cos(thetag/2.)
        Jem(2,2)=0
        Jem(2,3)=sqrt(2.)*Gm*(-(sqrt(ppCM(4) - m_p)*sqrt(pCM(4) +m_p)) + 
     1   sqrt(pCM(4) - m_p)*sqrt(ppCM(4) + m_p))*cos(thetag/2.)

        Jem(3,1)=(2*sqrt(2.)*(qCM(3) - qpCM(4))*Gm*
     1   (sqrt(ppCM(4) - m_p)*sqrt(pCM(4) + m_p) - 
     2   sqrt(pCM(4) - m_p)*sqrt(ppCM(4) +m_p))*sqrt(s)*cos(thetag/2.))/
     3   sqrt((qsq**2) - 4*s*tt) - (2*sqrt(2.)*qCM(3)*qpCM(1)*F2*
     4   (sqrt(pCM(4) - m_p)*sqrt(ppCM(4) - m_p) + 
     5   sqrt(pCM(4) + m_p)*sqrt(ppCM(4) +m_p))*sqrt(s)*sin(thetag/2.))/
     6   (m_p*sqrt((qsq**2) - 4*s*tt))

        Jem(3,2)=-((F2*(sqrt(pCM(4) - m_p)*sqrt(ppCM(4) - m_p) + 
     1   sqrt(pCM(4) + m_p)*sqrt(ppCM(4) + m_p))*
     2   (2*(m_p**2) + qsq + 2*s)*sqrt(-(s*tt))*sin(thetag/2.))/
     3   (m_p*sqrt(s)*sqrt((qsq**2) - 4*s*tt))) - 
     4   (8*Gm*(sqrt(ppCM(4) - m_p)*sqrt(pCM(4) + m_p) - 
     5   sqrt(pCM(4) - m_p)*sqrt(ppCM(4) + m_p))*s*sqrt(-tt)*
     6   (qpCM(1)*cos(thetag/2.) + (qCM(3) - qpCM(3))*sin(thetag/2.)))/
     7   (qsq*sqrt((qsq**2) - 4*s*tt))

        Jem(3,3)=(-2*sqrt(2.)*(qCM(3) - qpCM(4))*Gm*
     1   (sqrt(ppCM(4) - m_p)*sqrt(pCM(4) + m_p) - 
     2   sqrt(pCM(4) - m_p)*sqrt(ppCM(4) +m_p))*sqrt(s)*cos(thetag/2.))/
     3   sqrt((qsq**2) - 4*s*tt) + (2*sqrt(2.)*qCM(3)*qpCM(1)*F2*
     4   (sqrt(pCM(4) - m_p)*sqrt(ppCM(4) - m_p) + 
     5   sqrt(pCM(4) + m_p)*sqrt(ppCM(4) +m_p))*sqrt(s)*sin(thetag/2.))/
     6   (m_p*sqrt((qsq**2) - 4*s*tt))

        Jem(4,1)=sqrt(2.)*Gm*(sqrt(ppCM(4) - m_p)*sqrt(pCM(4) + m_p) + 
     1   sqrt(pCM(4) - m_p)*sqrt(ppCM(4) + m_p))*sin(thetag/2.)
        Jem(4,2)=0
        Jem(4,3)=sqrt(2.)*Gm*(sqrt(ppCM(4) - m_p)*sqrt(pCM(4) + m_p) + 
     1   sqrt(pCM(4) - m_p)*sqrt(ppCM(4) + m_p))*sin(thetag/2.)

        endif

c iexact=0:
c All quantities are evaluated at leading order in the 1/Q expansion 
c (see Belitsky, Mueller and Kirchner, Nucl.Phys. B629 (2002) 323-392,
c ArXiv:hep-ph/0112108v2)

      if (iexact.eq.0) then

        Jem(1,1)=-((sqrt(2.)*(F2 - Gm)*qpPerp*(-2 + xB))/(sqrt(1-xB)*xB))
        Jem(1,2)=-(((F2*((-2 + xB)**2) + 4*Gm*(-1 + xB))*
     1   sqrt( (qpPerp**2) + (m_p**2)*(xB**2))) / ((-1 + xB)*xB))
        Jem(1,3)=(sqrt(2.)*(F2 - Gm)*qpPerp*(-2 + xB))/(sqrt(1-xB)*xB)

        Jem(2,1)=(sqrt(2.)*Gm*m_p*xB)/sqrt(1-xB)
        Jem(2,2)=0
        Jem(2,3)=(sqrt(2.)*Gm*m_p*xB)/sqrt(1-xB)

        Jem(3,1)=-((sqrt(2.)*(F2*(qpPerp**2) + Gm*(m_p**2)*(xB**2)))/
     1     (m_p*sqrt(1 - xB)*xB))
        Jem(3,2)=-((F2*qpPerp*(-2 + xB)*
     1     sqrt((qpPerp**2) + (m_p**2)*(xB**2)))/(m_p*(-1 + xB)*xB))
        Jem(3,3)=(sqrt(2.)*(F2*(qpPerp**2) 
     1      + Gm*(m_p**2)*(xB**2)))/(m_p*sqrt(1-xB)*xB)

        Jem(4,1)=(sqrt(2.)*Gm*qpPerp)/sqrt(1-xB)
        Jem(4,2)=0
        Jem(4,3)=(sqrt(2.)*Gm*qpPerp)/sqrt(1-xB)
      
      endif
         
c----------------------------- BH cross sections -------------------------------*
c need to add +1 to all indices, since fortran starts at 1, but c++ starts at 0.

      SigmaBHPol0(1)=(-32*(Ge**2)*(m_p**2)*(3*(m_p**8)*tt + 
     1    3*(m_p**6)*(qsq**2 + qsq*tt - 4*s*tt) + 
     2    (m_p**2)*(3*qsq**2*((qsq + s)**2) - 
     3    (qsq + s)*(qsq**2 + 3*qsq*s + 12*(s**2))*tt + 
     4    2*(qsq**2 + qsq*s - 3*(s**2))*(tt**2)) + 
     5    ((qsq + s)**2)*tt*(3*s*(s + t) + qsq*(3*s + tt)) + 
     6    (m_p**4)*(4*(qsq**3) + qsq*tt*(3*s + tt) - 
     7    qsq**2*(6*s + tt) + 3*s*tt*(6*s + tt))) + 
     8    4*(Gm**2)*tt*(6*(m_p**8)*tt + tt*(3*((qsq + s)**2))*
     9    (qsq**2 + 2*qsq*s + 2*(s**2)) + 2*s*(qsq + s)*(2*qsq + 3*s)*t + 
     1    (3*qsq**2 + 4*qsq*s + 3*(s**2))*(tt**2)) - 
     2    2*(m_p**6)*(3*qsq**2 - 11*qsq*t + 6*tt*(2*s + tt)) + 
     3    (m_p**4)*(-8*(qsq**3) - 26*qsq*tt*(s + tt) + 
     4    qsq**2*(12*s + 25*tt) + 3*tt*(12*(s**2) + 10*s*t + (tt**2))) - 
     5    2*(m_p**2)*(3*(qsq**4) + (qsq**3)*(6*s - 5*t) + 
     6    3*s*tt*((2*s + t)**2) + qsq*tt*(7*(s**2) + 2*s*tt - 3*(tt**2)) + 
     7    qsq**2*(3*(s**2) - 5*s*tt + 7*(tt**2)))))/
     8    (((m_p**4) + 2*(m_p**2)*(qsq - s) + ((qsq + s)**2))*
     9    (4*(m_p**2) - tt))

      SigmaBHPol0(2)=(-32*(Ge**2)*(m_p**2)*((m_p**8)*tt + 
     1    (m_p**6)*(qsq**2 + qsq*tt - 4*s*tt) + 
     2    (m_p**2)*(qsq**2*((qsq + s)**2) + 
     3    (qsq - s)*(qsq + s)*(5*qsq + 4*s)*tt - 
     4    2*(qsq**2 + qsq*s + (s**2))*(tt**2)) + 
     5    ((qsq + s)**2)*tt*(qsq*(s - tt) + s*(s + tt)) + 
     6    (m_p**4)*(-4*(qsq**3) + qsq*(s - tt)*tt + s*tt*(6*s + tt) + 
     7    qsq**2*(-2*s + 5*tt))) + 4*(Gm**2)*tt*(2*(m_p**8)*tt + 
     8    tt*(((qsq + s)**2)*(qsq**2 + 2*qsq*s + 2*(s**2)) + 
     9    2*s*(-2*qsq**2 - qsq*s + (s**2))*tt + 
     1    (qsq**2 - 4*qsq*s + (s**2))*(tt**2)) - 
     2    2*(m_p**6)*(qsq**2 - 9*qsq*tt + 2*tt*(2*s + tt)) + 
     3    (m_p**4)*(8*(qsq**3) - 2*qsq*tt*(15*s + 7*tt) + 
     4    qsq**2*(4*s + 19*tt) +tt*(12*(s**2) + 10*s*tt + (tt**2)))\
     5    - 2*(m_p**2)*((qsq**4) + (qsq**3)*(2*s + tt) + 
     6    s*tt*((2*s+tt)**2) - qsq*tt*(3*(s**2) +10*s*tt +(tt**2)) + 
     7    qsq**2*((s**2) - 7*s*tt + 5*(tt**2)))))/
     8    (((m_p**4) + 2*(m_p**2)*(qsq - s) + ((qsq + s)**2))* 
     9    (4*(m_p**2) - tt))

      SigmaBHPol0(3)=(-64*(Ge**2)*(m_p**2)*Q*qpPerp*((m_p**2) -qsq -s)*
     1    (2*(m_p**2)*qsq - ((m_p**2) + qsq + s)*tt) + 
     2    16*(Gm**2)*Q*qpPerp*tt*((m_p**4)*(-2*qsq + 3*tt) + 
     3    tt*(qsq*(s - tt) + s*(s + tt)) + 
     4    (m_p**2)*(2*qsq**2 - tt*(4*s + tt) + qsq*(2*s + 5*tt))))/
     5    (sqrt((m_p**4) + 2*(m_p**2)*(qsq - s) + 
     6    ((qsq + s)**2))*(4*(m_p**2) - tt))

      SigmaBHPol0(4)=(-64*(Ge**2)*(m_p**4)*qsq*
     1    ((m_p**4)*tt + s*tt*(qsq + s + tt) + 
     2    (m_p**2)*(qsq**2 + qsq*tt - 2*s*tt)) - 
     3    8*(Gm**2)*qsq*(2*(m_p**2) - tt)*tt*
     4    ((m_p**4)*tt + s*tt*(qsq + s + tt) + 
     5    (m_p**2)*(qsq**2 + qsq*tt - 2*s*tt)))/
     6    (((m_p**4) + 2*(m_p**2)*(qsq - s) + ((qsq + s)**2))*
     7    (4*(m_p**2) - tt))

      SigmaBHPolX(1)=(64*Ge*Gm*m_p*qpPerp*(-(qsq*(qsq + s)) + 
     1    (m_p**2)*(qsq - tt) + s*tt)*
     2    (2*(m_p**2)*qsq - ((m_p**2) + qsq + s)*tt) + 
     3    32*(Gm**2)*m_p*qpPerp*tt*(-(qsq**3) - 3*qsq**2*s - 
     4    2*(m_p**4)*(qsq - tt) + s*tt*(2*s + tt) - 
     5    qsq*(2*(s**2) + (tt**2)) - 
     6    (m_p**2)*(qsq**2 - 4*qsq*(s + tt) + tt*(4*s + tt))))/
     7    (sqrt((m_p**4) + 2*(m_p**2)*(qsq - s) + 
     8    ((qsq + s)**2))*(4*(m_p**2) - tt))

      SigmaBHPolX(2)=(32*Ge*Gm*m_p*Q*(-2*(m_p**2)*qsq + 
     1    ((m_p**2) + qsq + s)*tt) *(4*(m_p**2)*qsq**2 + 
     2    (2*(m_p**2) + qsq - 2*s)*((m_p**2) - qsq - s)*tt + 
     3    ((m_p**2) + qsq + 3*s)*(tt**2)) + 
     4    64*(Gm**2)*m_p*Q*((m_p**2) - s - tt)*tt*
     5    ((m_p**4)*tt + s*tt*(qsq + s + tt) + 
     6    (m_p**2)*(qsq**2 + qsq*tt - 2*s*tt)))/
     7    (((m_p**4) + 2*(m_p**2)*(qsq - s) + ((qsq + s)**2))*
     8    (4*(m_p**2) - tt))

      SigmaBHPolY=32*Ge*Gm*m_p*Q*(qsq - tt)*tt

      SigmaBHPolZ(1)=(-128*Ge*Gm*(m_p**2)*(-(qsq*(qsq + s)) + 
     1    (m_p**2)*(qsq - tt) + s*tt)* ((m_p**4)*tt + s*tt*(qsq + s + tt) + 
     2    (m_p**2)*(qsq**2 + qsq*tt - 2*s*tt)) + 
     3    16*(Gm**2)*tt*(2*(m_p**2)*qsq - ((m_p**2) + qsq + s)*tt)*
     4    ((qsq**3) + 3*qsq**2*s + 2*(m_p**4)*(qsq - tt) - 
     5    s*tt*(2*s + tt) + qsq*(2*(s**2) + (tt**2)) + 
     6    (m_p**2)*(qsq**2 - 4*qsq*(s + tt) + tt*(4*s + tt))))/
     7    (((m_p**4) + 2*(m_p**2)*(qsq - s) + ((qsq + s)**2))*
     8    (4*(m_p**2) - tt))

      SigmaBHPolZ(2)=(64*Ge*Gm*(m_p**2)*Q*qpPerp*(-4*(m_p**2)*qsq**2 - 
     1    (2*(m_p**2) + qsq - 2*s)*((m_p**2) - qsq - s)*tt - 
     2    ((m_p**2) + qsq + 3*s)*(tt**2)) + 
     3    32*(Gm**2)*Q*qpPerp*tt*(-(m_p**2) + s + tt)*
     4    ((qsq + s)*tt + (m_p**2)*(-2*qsq + tt)))/
     5    (sqrt((m_p**4) + 2*(m_p**2)*(qsq - s) + 
     6    ((qsq + s)**2))*(4*(m_p**2) - tt))

      if (debug(2)) then 
         write(6,*)' F1 ',F1
         write(6,*)' F2 ',F2
         write(6,*)' Ge ',Ge
         write(6,*)' Gm ',Gm

         do i=1,4
            write(6,*)' Jem ',i,(Jem(i,j),j=1,3)
         enddo          
          
         write(6,*)' SigmaBHPol0 ',(SigmaBHPol0(i),i=1,4)
         write(6,*)' SigmaBHPolX ',(SigmaBHPolX(i),i=1,2)
         write(6,*)' SigmaBHPolY ',SigmaBHPolY
         write(6,*)' SigmaBHPolZ ',(SigmaBHPolZ(i),i=1,2)
      endif
     
c------------------------- MakeVCSAndInterfCrossSections -----------------------*
c Computes all the stuff to evaluate the cross section assuming the hadronic    |
c helicity amplitudes are given, i.e. initializes :                             |
c   - the helicity amplitudes Mvcs,                                             |
c   - the expansion coefficients Ur,                                            |
c   - the SigmaVCSPol's and SigmaIPol's.                                        |
c-------------------------------------------------------------------------------*
 
c---------- Harmonic expansion coefficients of the VCS cross section -----------*
c need to add +1 to all indices, since fortran starts at 1, but c++ starts at 0.

c iexact=1:
c No approximations apart those in the computation of the hadronic helicity 
c amplitudes in terms of the Compton Form Factors.

      if (iexact.eq.1) then

       Ur(1)=-(((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt)*
     1    ((qsq**2) + 4*qCM(4)*(4*qCM(4) - 3*qCM(3))*
     2    (qpPerp**2) + (qCM(4) + qCM(3))* (qCM(4) + qpCM(4))*tt + 
     3    qsq*((qCM(4) - qpCM(4))* (qCM(4) + qCM(3) + 2*qpCM(4)) + 
     4    16*(qpPerp**2) + tt)))/
     5    (64.*qCM(3)*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(2)=(((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt)*
     1    ((qsq**2) - 4*qCM(4)*qCM(3)*(qpPerp**2) + 
     2    (qCM(4) - qCM(3))*(qCM(4) + qpCM(4))*tt + 
     3    qsq*((qCM(4) - qpCM(4))*(qCM(4) - qCM(3) + 2*qpCM(4)) + tt)))/
     4    (64.*qCM(3)*qsq*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(3)=(((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt)*
     1    ((qsq**2) + 4*qCM(4)*qCM(3)*(qpPerp**2) + 
     2    (qCM(4) + qCM(3))*(qCM(4) + qpCM(4))*tt + 
     3    qsq*((qCM(4) - qpCM(4))* (qCM(4) + qCM(3) +2*qpCM(4)) + tt)))/
     4    (64.*qCM(3)*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(4)=(qpPerp*((qsq**2) + qsq*(12*(qCM(4)**2) -16*qCM(4)*qCM(3) - 
     1    22*qCM(4)*qpCM(4) + 26*qCM(3)*qpCM(4) - 7*tt) + 
     2    4*qCM(4)*(qCM(4) - 4*qCM(3))*tt)*
     3    ((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt))
     4    /(128.*qCM(3)*qsq*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(5)=(3*qpPerp*(2*(qCM(4) + qCM(3))*qpCM(4) + qsq + tt)*
     1    ((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt))
     2    /(128.*qCM(3)*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(6)=(qpPerp*(5*(qsq**2) + qsq*
     1    (2*(6*(qCM(4)**2) - 7*qCM(4)*qpCM(4) + 
     2    qCM(3)*qpCM(4)) - 3*t) + 4*(qCM(4)**2)*tt)*
     3    ((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt))
     4    /(128.*qCM(3)*qsq*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(7)=-(((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt)*
     1    ((qsq**2) + qsq*((qCM(4) - qpCM(4))*
     2    (qCM(4) - qCM(3) + 2*qpCM(4)) + 2*(qpPerp**2) + tt) + 
     3    (qCM(4) - qCM(3))* 
     4    (2*qCM(4)*(qpPerp**2) + (qCM(4) + qpCM(4))*tt)))/
     5    (32.*qCM(3)*qsq*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(8)=-(qpPerp*(qCM(4)*(3*qCM(4) - qCM(3) - 2*qpCM(4))*
     1    qsq + 2*(qsq**2) + qCM(4)*(qCM(4) - qCM(3))*tt)*
     2    ((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt)
     3    )/(16.*qCM(3)*qsq*sqrt(s)*sqrt(-tt)* sqrt((qsq**2) - 4*s*tt))

       Ur(9)=(qpPerp*sqrt(s)*((4*(qCM(4)**2) - 
     1    3*qCM(4)*(qCM(3) - qpCM(4)) - 4*qCM(3)*qpCM(4))*tt + 
     2    qsq*(-3*(qCM(4) + qCM(3) - 2*qpCM(4))* 
     3    (qCM(4) - qpCM(4)) + 7*tt)))/
     4    (4.*sqrt(2.)*qCM(3)*sqrt((qsq**2) - 4*s*tt))

       Ur(10)=-(qpPerp*sqrt(s)*(qCM(4)*(qCM(3) + qpCM(4))*tt + 
     1    qsq*(-((qCM(4) - qCM(3) - 2*qpCM(4))*
     2    (qCM(4) - qpCM(4))) + tt)))/
     3    (4.*sqrt(2.)*qCM(3)*qsq*sqrt((qsq**2) - 4*s*tt))

       Ur(11)=(qpPerp*sqrt(s)*(qCM(4)*(-qCM(3) + qpCM(4))*tt + 
     1    qsq*(-((qCM(4) + qCM(3) - 2*qpCM(4))*
     2    (qCM(4) - qpCM(4))) + tt)))/
     3    (4.*sqrt(2.)*qCM(3)*sqrt((qsq**2) - 4*s*tt))

       Ur(12)=(sqrt(s)*(3*(qsq**3) + (qsq**2)*
     1    (3*(qCM(4)**2) - 7*qCM(4)*qCM(3) + 35*qCM(4)*qpCM(4) + 
     2    3*qCM(3)*qpCM(4) - 22*(qpCM(4)**2) + 12*(qpPerp**2) + 3*tt) + 
     3    8*qCM(4)*(4*qCM(4) - qCM(3))* (qCM(4) + qpCM(4))*
     4    (qCM(4)*(qpPerp**2) + qpCM(4)*tt) + 
     5    qsq*(8*(4*qCM(4) - qCM(3))*qpCM(4)* 
     6    ((qCM(4)**2) - (qpCM(4)**2)) + 
     7    2*(22*(qCM(4)**2) + 7*qCM(3)*qpCM(4) + 
     8    qCM(4)*(-15*qCM(3) +16*qpCM(4)))*(qpPerp**2) +(3*(qCM(4)**2) - 
     9    7*qCM(4)*(qCM(3) - 5*qpCM(4)) +  
     1    qpCM(4)*(-11*qCM(3) + 16*qpCM(4)))*tt)))/
     2    (16.*sqrt(2.)*qCM(3)*qsq*sqrt((qsq**2) - 4*s*tt))

       Ur(13)=(-3*sqrt(s)*((qsq**2) + 2*(2*(qCM(4)**2) - 
     1    qCM(4)*qCM(3) + qCM(3)*qpCM(4))* 
     2    (qpPerp**2) + (qCM(4) - qCM(3))* (qCM(4) + qpCM(4))*tt + 
     3    qsq*((qCM(4) - qpCM(4))* (qCM(4) - qCM(3) + 2*qpCM(4)) + 
     4    4*(qpPerp**2) + tt)))/
     5    (16.*sqrt(2.)*qCM(3)*qsq*sqrt((qsq**2) - 4*s*tt))

       Ur(14)=-(sqrt(s)*((qsq**3) + (qsq**2)*
     1    ((qCM(4)**2) + 3*qCM(4)*qCM(3) + qCM(4)*qpCM(4) 
     2    + qCM(3)*qpCM(4) - 2*(qpCM(4)**2) + 4*(qpPerp**2) + tt) + 
     3    8*qCM(4)*qCM(3)*(qCM(4) + qpCM(4))*
     4    (qCM(4)*(qpPerp**2) + qpCM(4)*tt) + qsq*(8*qCM(3)*qpCM(4)*
     5    ((qCM(4)**2) - (qpCM(4)**2)) + (4*(qCM(4)**2) 
     6    + 22*qCM(4)*qCM(3) - 6*qCM(3)*qpCM(4))*(qpPerp**2) + 
     7    ((qCM(4)**2) + 7*qCM(3)*qpCM(4) 
     8    + qCM(4)*(3*qCM(3) + qpCM(4)))*tt)))/
     9    (16.*sqrt(2.)*qCM(3)*qsq*sqrt((qsq**2) - 4*s*tt))

       Ur(15)=((qCM(4) - qpCM(4))*qpPerp*sqrt(s)*
     1    ((-qCM(4) + qCM(3) + 2*qpCM(4))*qsq + (-qCM(4) + qCM(3))*tt))/
     2    (4.*sqrt(2.)*qCM(3)*qsq*sqrt((qsq**2) - 4*s*tt))

       Ur(16)=-(sqrt(s)*((qsq**3) + (qsq**2)*
     1    ((qCM(4)**2) + 3*qCM(4)*qpCM(4) - qCM(3)*qpCM(4) 
     2    - 3*(qpCM(4)**2) + 4*(qpPerp**2) + tt) + 
     3    2*qCM(4)*(qCM(4) - qCM(3))* (qCM(4) + qpCM(4))*
     4    (qCM(4)*(qpPerp**2) + qpCM(4)*tt) + 
     5    qsq*(2*(qCM(4) -qCM(3))*qpCM(4)* ((qCM(4)**2) -(qpCM(4)**2)) + 
     6    2*qCM(4)*(3*qCM(4) - 2*qCM(3) + qpCM(4))*
     7    (qpPerp**2) + ((qCM(4)**2) +3*qCM(4)*qpCM(4) -qCM(3)*qpCM(4) + 
     8    (qpCM(4)**2))*tt)))/
     9    (2.*sqrt(2.)*qCM(3)*qsq*sqrt((qsq**2) - 4*s*tt))

       Ur(17)=(qpPerp*((qCM(4) - qCM(3) - 2*qpCM(4))*qsq + 
     1    qCM(4)*tt))/(8.*sqrt(2.)*qCM(3)*qsq)

       Ur(18)=-((3*qCM(4) - 7*qCM(3) + 10*qpCM(4))*(qsq**2) + 
     1    8*qCM(4)*(4*qCM(4) - qCM(3))*(qCM(4)*(qpPerp**2)+ qpCM(4)*t) + 
     2    qsq*(8*(4*qCM(4) - qCM(3))* (qCM(4) - qpCM(4))*qpCM(4) + 
     3    (32*qCM(4) - 6*qCM(3))*(qpPerp**2) + 
     4    (3*qCM(4) - 7*qCM(3) + 16*qpCM(4))*tt))/
     5    (32.*sqrt(2.)*qCM(3)*qsq)

       Ur(19)=(3*((-qCM(4) + qCM(3) + 2*qpCM(4))*qsq + 
     1    2*qCM(3)*(qpPerp**2) + (-qCM(4) + qCM(3))*tt))/
     2    (32.*sqrt(2.)*qCM(3)*qsq)

       Ur(20)=((qCM(4) + 3*qCM(3) - 2*qpCM(4))*(qsq**2) + 
     1    qsq*(-2*qCM(3)* (4*qpCM(4)*(-qCM(4) +qpCM(4)) + (qpPerp**2)) + 
     2    (qCM(4) + 3*qCM(3))*tt) + 
     3    8*qCM(4)*qCM(3)*(qCM(4)*(qpPerp**2) + qpCM(4)*tt)
     4    )/(32.*sqrt(2.)*qCM(3)*qsq)

       Ur(21)=(qpPerp*((qCM(4) + qCM(3) - 2*qpCM(4))*qsq + 
     1    (qCM(4) - 3*qCM(3))*tt))/(8.*sqrt(2.)*qCM(3))

       Ur(22)=(qpPerp*((-qCM(4) + qCM(3) + 2*qpCM(4))*qsq + 
     1    (-qCM(4) + qCM(3))*t))/(8.*sqrt(2.)*qCM(3)*qsq)

       Ur(23)=(qpPerp*(3*(qCM(4) + qCM(3) - 2*qpCM(4))*qsq + 
     1    (3*qCM(4) - qCM(3))*tt))/(8.*sqrt(2.)*qCM(3))

       Ur(24)=((qCM(4) - qpCM(4))*(qsq**2) + 2*qCM(4)*(qCM(4) - qCM(3))*
     1    (qCM(4)*(qpPerp**2) + qpCM(4)*tt) + 
     2    qsq*(2*(qCM(4) - qCM(3))*(qCM(4) - qpCM(4)\
     3    )*qpCM(4) + 2*(qCM(4) + qCM(3))*
     4    (qpPerp**2) + (qCM(4) + qpCM(4))*tt))/(4.*sqrt(2.)*qCM(3)*qsq)

       Ur(25)=(qpPerp*((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 
     1    4*s*tt)*(3*(qsq**2) + 2*(qCM(4)**2)*tt - 
     2    qsq*(6*qCM(4)*(-qCM(4) + qpCM(4)) + tt)))/
     3    (32.*sqrt(2.)*qCM(3)*qsq*sqrt(s)*sqrt(-tt)*
     4    sqrt((qsq**2) - 4*s*tt))

       Ur(26)=-(qpPerp*(2*qCM(4)*qpCM(4) + qsq + tt)*
     1    ((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt))/
     2    (32.*sqrt(2.)*qCM(3)*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) -4*s*tt))

       Ur(27)=((3*(qCM(4) - qpCM(4))*qsq + 4*qCM(4)*(qpPerp**2) + 
     1    3*(qCM(4) + qpCM(4))*tt)*
     2    ((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt))
     3    /(16.*sqrt(2.)*qsq*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(28)=-(((qCM(4) - qpCM(4))*qsq + 4*qCM(4)*(qpPerp**2) + 
     1    (qCM(4) + qpCM(4))*tt)*
     2    ((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt)
     3    )/(16.*sqrt(2.)*qsq*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(29)=-(qCM(4)*(qpPerp**2)*((qsq**2) + 
     1    2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt))/
     2    (4.*sqrt(2.)*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(30)=-(sqrt(s)*((qCM(4) + qpCM(4))*(qsq**2) + 
     1    4*qCM(4)*(qCM(4) +qpCM(4))* (qCM(4)*(qpPerp**2) +qpCM(4)*tt) + 
     2    qsq*(4*(qCM(4)**2)*qpCM(4) - 4*(qpCM(4)**3) + 
     3    2*(5*qCM(4) - qpCM(4))*(qpPerp**2) + qCM(4)*tt + 
     4    3*qpCM(4)*t)))/(8.*qsq*sqrt((qsq**2) - 4*s*tt))

       Ur(31)=-(sqrt(s)*((qCM(4) - qpCM(4))*qsq + 
     1    2*(qCM(4) - qpCM(4))*(qpPerp**2) + (qCM(4) + qpCM(4))*tt))/
     2    (8.*qsq*sqrt((qsq**2) - 4*s*tt))

       Ur(32)=-(qpPerp*sqrt(s)*(qCM(4)*(2*qCM(4) - qpCM(4))*tt + 
     1    qsq*((qCM(4)**2) - 3*qCM(4)*qpCM(4) + 2*(qpCM(4)**2) + tt)))/
     2    (2.*qCM(3)*qsq*sqrt((qsq**2) - 4*s*tt))

       Ur(33)=-(qpPerp*sqrt(s)*(qCM(4)*qpCM(4)*tt + 
     1    qsq*(-(qCM(4)**2) + 3*qCM(4)*qpCM(4) - 2*(qpCM(4)**2) + tt)))/
     2    (2.*qCM(3)*qsq*sqrt((qsq**2) - 4*s*tt))

       Ur(34)=-(qpPerp*sqrt(s)*(qCM(4)*(qCM(4) + qpCM(4))*tt + 
     1    qsq*(-(qCM(4)**2) + 3*qCM(4)*qpCM(4) -2*(qpCM(4)**2) +2*tt)))/
     2    (2.*qCM(3)*qsq*sqrt((qsq**2) - 4*s*tt))

       Ur(35)=((qCM(4) - 2*qpCM(4))*(Q**5) + qCM(4)*(Q**3)*tt)/
     1    (16.*qCM(3)*(Q**5))
       Ur(36)=(qpPerp*(qsq + 2*tt))/(4.*qsq)
       Ur(37)=-qpPerp/4.

       Ur(38)=(qCM(4)*(qsq**2) + 4*(qCM(4)**2)*
     1    (qCM(4)*(qpPerp**2) + qpCM(4)*tt) + 
     2    qsq*(4*qCM(4)*(qCM(4) - qpCM(4))*qpCM(4) + 
     3    4*qCM(4)*(qpPerp**2) + (qCM(4) + 2*qpCM(4))*tt))/
     4    (8.*qCM(3)*qsq)
       Ur(39)=(qpPerp*(-qsq + tt))/(4.*qsq)

       Ur(40)=-(((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt)*
     1    ((qsq**2) + 4*qCM(4)*(4*qCM(4) + 3*qCM(3))*
     2    (qpPerp**2) + (qCM(4) - qCM(3))* (qCM(4) + qpCM(4))*tt + 
     3    qsq*((qCM(4) - qpCM(4))* (qCM(4) - qCM(3) + 2*qpCM(4)) + 
     4    16*(qpPerp**2) + tt)))/
     5    (64.*qCM(3)*qsq*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(41)=-(qpPerp*((qsq**2) + qsq*
     1    (12*(qCM(4)**2) + 16*qCM(4)*qCM(3) - 
     2    22*qCM(4)*qpCM(4) - 26*qCM(3)*qpCM(4) - 7*tt)\
     3    + 4*qCM(4)*(qCM(4) + 4*qCM(3))*tt)*
     4    ((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt)
     5    )/(128.*qCM(3)*qsq*sqrt(s)*sqrt(-tt)*
     6    sqrt((qsq**2) - 4*s*tt))

       Ur(42)=(-3*qsq*qpPerp*(2*(qCM(4) - qCM(3))*qpCM(4) + qsq + tt)*
     1    ((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt))/
     2    (128.*qCM(3)*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(43)=-(qpPerp*((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 
     1    4*s*tt)*(5*(qsq**2) + 4*(qCM(4)**2)*tt - 
     2    qsq*(2*(-6*(qCM(4)**2) + 7*qCM(4)*qpCM(4) + 
     3    qCM(3)*qpCM(4)) + 3*tt)))/
     4    (128.*qCM(3)*qsq*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(44)=-(((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt)*
     1    ((qsq**2) + qsq*((qCM(4) - qpCM(4))*
     2    (qCM(4) + qCM(3) + 2*qpCM(4)) + 2*(qpPerp**2) + tt) + 
     3    (qCM(4) + qCM(3))* 
     4    (2*qCM(4)*(qpPerp**2) + (qCM(4) + qpCM(4))*tt)))/
     5    (32.*qCM(3)*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(45)=(qpPerp*(qCM(4)*(3*qCM(4) + qCM(3) - 2*qpCM(4))*
     1    qsq + 2*(qsq**2) + qCM(4)*(qCM(4) + qCM(3))*tt)*
     2    ((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 
     3    4*s*tt))/(16.*qCM(3)*qsq*sqrt(s)*sqrt(-tt)*
     4    sqrt((qsq**2) - 4*s*tt))

       Ur(46)=-(qpPerp*sqrt(s)*((4*(qCM(4)**2) + 4*qCM(3)*qpCM(4) + 
     1    3*qCM(4)*(qCM(3) + qpCM(4)))*tt + 
     2    qsq*(-3*(qCM(4) - qCM(3) - 2*qpCM(4))*
     3    (qCM(4) - qpCM(4)) + 7*tt)))/
     4    (4.*sqrt(2.)*qCM(3)*qsq*sqrt((qsq**2) - 4*s*tt))

       Ur(47)=(sqrt(s)*(3*(qsq**3) + (qsq**2)*
     1    (3*(qCM(4)**2) + 7*qCM(4)*qCM(3) + 
     2    35*qCM(4)*qpCM(4) - 3*qCM(3)*qpCM(4) - 
     3    22*(qpCM(4)**2) + 12*(qpPerp**2) + 3*tt) + 
     4    8*qCM(4)*(4*qCM(4) + qCM(3))* (qCM(4) + qpCM(4))*
     5    (qCM(4)*(qpPerp**2) + qpCM(4)*tt) + 
     6    qsq*(8*(4*qCM(4) + qCM(3))*qpCM(4)* 
     7    ((qCM(4)**2) - (qpCM(4)**2)) + 
     8    2*(22*(qCM(4)**2) - 7*qCM(3)*qpCM(4) + 
     9    qCM(4)*(15*qCM(3) + 16*qpCM(4)))*(qpPerp**2)\
     1    + (3*(qCM(4)**2) + 7*qCM(4)*(qCM(3) + 5*qpCM(4)) + 
     2    qpCM(4)*(11*qCM(3) + 16*qpCM(4)))*tt)))/
     3    (16.*sqrt(2.)*qCM(3)*(qsq**2)*sqrt((qsq**2) - 4*s*tt))

       Ur(48)=(-3*qsq*sqrt(s)*((qsq**2) + 
     1    2*(2*(qCM(4)**2) +qCM(4)*qCM(3) -qCM(3)*qpCM(4))*(qpPerp**2) + 
     2    (qCM(4) + qCM(3))*(qCM(4) + qpCM(4))*tt + 
     3    qsq*((qCM(4) - qpCM(4))* (qCM(4) + qCM(3) + 2*qpCM(4)) + 
     4    4*(qpPerp**2) + tt)))/ 
     5    (16.*sqrt(2.)*qCM(3)*sqrt((qsq**2) - 4*s*tt))

       Ur(49)=-(sqrt(s)*((qsq**3) + (qsq**2)*
     1    ((qCM(4)**2) -3*qCM(4)*qCM(3) +qCM(4)*qpCM(4) -qCM(3)*qpCM(4) - 
     2    2*(qpCM(4)**2) + 4*(qpPerp**2) + tt) - 
     3    8*qCM(4)*qCM(3)*(qCM(4) + qpCM(4))*
     4    (qCM(4)*(qpPerp**2) + qpCM(4)*tt) + qsq*(8*qCM(3)*qpCM(4)*
     5    (-(qCM(4)**2) + (qpCM(4)**2)) + (4*(qCM(4)**2) 
     6    - 22*qCM(4)*qCM(3) + 6*qCM(3)*qpCM(4))*(qpPerp**2) + 
     7    ((qCM(4)**2) - 7*qCM(3)*qpCM(4) + 
     8    qCM(4)*(-3*qCM(3) + qpCM(4)))*tt)))/
     9    (16.*sqrt(2.)*qCM(3)*(qsq**2)*sqrt((qsq**2) - 4*s*tt))

       Ur(50)=((qCM(4) - qpCM(4))*qpPerp*sqrt(s)*
     1    ((qCM(4) + qCM(3) - 2*qpCM(4))*qsq + (qCM(4) + qCM(3))*tt))/
     2    (4.*sqrt(2.)*qCM(3)*sqrt((qsq**2) - 4*s*tt))

       Ur(51)=-(sqrt(s)*((qsq**3) + (qsq**2)*
     1    ((qCM(4)**2) + 3*qCM(4)*qpCM(4) + qCM(3)*qpCM(4) 
     2    - 3*(qpCM(4)**2) + 4*(qpPerp**2) + tt) + 
     3    2*qCM(4)*(qCM(4) + qCM(3))* (qCM(4) + qpCM(4))*
     4    (qCM(4)*(qpPerp**2) + qpCM(4)*tt) + 
     5    qsq*(2*(qCM(4) +qCM(3))*qpCM(4)* ((qCM(4)**2) -(qpCM(4)**2)) + 
     6    2*qCM(4)*(3*qCM(4) + 2*qCM(3) + qpCM(4))*
     7    (qpPerp**2) + ((qCM(4)**2) + 3*qCM(4)*qpCM(4) + 
     8    qpCM(4)*(qCM(3) + qpCM(4)))*tt)))/
     9    (2.*sqrt(2.)*qCM(3)*(qsq**2)*sqrt((qsq**2) - 4*s*tt))

       Ur(52)=(qpPerp*((qCM(4) + qCM(3) - 2*qpCM(4))*qsq + 
     1    qCM(4)*tt))/(8.*sqrt(2.)*qCM(3))

       Ur(53)=((3*qCM(4) + 7*qCM(3) + 10*qpCM(4))*(qsq**2) + 
     1    8*qCM(4)*(4*qCM(4) + qCM(3))* 
     2    (qCM(4)*(qpPerp**2) + qpCM(4)*tt) +qsq*(8*(4*qCM(4) + qCM(3))*
     3    (qCM(4) - qpCM(4))*qpCM(4) + 
     4    (32*qCM(4) + 6*qCM(3))*(qpPerp**2) + 
     5    (3*qCM(4) + 7*qCM(3) + 16*qpCM(4))*tt))/
     6    (32.*sqrt(2.)*qCM(3)*(qsq**2))

       Ur(54)=(3*qsq*((qCM(4) + qCM(3) - 2*qpCM(4))*qsq + 
     1    2*qCM(3)*(qpPerp**2) + (qCM(4) + qCM(3))*tt))/
     2    (32.*sqrt(2.)*qCM(3))

       Ur(55)=((-qCM(4) + 3*qCM(3) + 2*qpCM(4))*(qsq**2) -qsq*(2*qCM(3)*
     1    (4*qpCM(4)*(-qCM(4) + qpCM(4)) + (qpPerp**2)) + 
     2    (qCM(4) - 3*qCM(3))*tt) + 
     3    8*qCM(4)*qCM(3)*(qCM(4)*(qpPerp**2) + qpCM(4)*tt)
     4    )/(32.*sqrt(2.)*qCM(3)*(qsq**2))

       Ur(56)=(qpPerp*((qCM(4) - qCM(3) - 2*qpCM(4))*qsq + 
     1    (qCM(4) + 3*qCM(3))*tt))/(8.*sqrt(2.)*qCM(3)*qsq)

       Ur(57)=-(qpPerp*((qCM(4) + qCM(3) - 2*qpCM(4))*qsq + 
     1   (qCM(4) + qCM(3))*tt))/(8.*sqrt(2.)*qCM(3))

       Ur(58)=(qpPerp*(3*(qCM(4) - qCM(3) - 2*qpCM(4))*qsq + 
     1    (3*qCM(4) + qCM(3))*tt))/(8.*sqrt(2.)*qCM(3)*qsq)

       Ur(59)=-((qCM(4) -qpCM(4))*(qsq**2) + 2*qCM(4)*(qCM(4) + qCM(3))*
     1    (qCM(4)*(qpPerp**2) + qpCM(4)*tt) + 
     2    qsq*(2*(qCM(4) + qCM(3))*(qCM(4) - qpCM(4)\
     3    )*qpCM(4) + 2*(qCM(4) - qCM(3))*(qpPerp**2) + 
     4    (qCM(4) + qpCM(4))*tt))/(4.*sqrt(2.)*qCM(3)*(qsq**2))

       Ur(60)=-(qpPerp*((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 
     1    4*s*tt)*(11*(qsq**2) + 4*qCM(4)*(qCM(4) - 4*qCM(3))*tt + 
     2    qsq*(2*(6*(qCM(4)**2) + 3*qCM(3)*qpCM(4) - 
     3    qCM(4)*(8*qCM(3) + qpCM(4))) + 3*tt)))/
     4    (128.*qCM(3)*qsq*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(61)=-(qpPerp*((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 
     1   4*s*tt)*(7*(qsq**2) + 4*(qCM(4)**2)*tt - 
     2   qsq*(2*(-6*(qCM(4)**2) + 5*qCM(4)*qpCM(4) + 
     3   qCM(3)*qpCM(4)) + tt)))/
     4   (128.*qCM(3)*qsq*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(62)=-(((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt)*
     1    ((qsq**2) + 2*qCM(4)*(3*qCM(4) - qCM(3))*
     2    (qpPerp**2) + (qCM(4) + qCM(3))* (qCM(4) + qpCM(4))*tt + 
     3    qsq*((qCM(4) - qpCM(4))* (qCM(4) + qCM(3) + 2*qpCM(4)) + 
     4    6*(qpPerp**2) + tt)))/
     5    (32.*qCM(3)*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(63)=(((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt)*
     1    ((qsq**2) - 2*qCM(4)*(qCM(4) - 3*qCM(3))*
     2    (qpPerp**2) + (qCM(4) + qCM(3))* (qCM(4) + qpCM(4))*tt + 
     3    qsq*((qCM(4) - qpCM(4))* (qCM(4) + qCM(3) + 2*qpCM(4)) - 
     4    2*(qpPerp**2) + tt)))/
     5    (32.*qCM(3)*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(64)=(qpPerp*((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 
     1    4*s*tt)*((qsq**2) + qCM(4)*(qCM(4) - qCM(3))*tt - 
     2    qsq*(-3*(qCM(4)**2) - 2*qCM(3)*qpCM(4) + 
     3    qCM(4)*(qCM(3) + 4*qpCM(4)) + tt)))/
     4    (16.*qCM(3)*qsq*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(65)=-(sqrt(s)*(13*(qsq**3) + (qsq**2)*
     1    (13*(qCM(4)**2) + 3*qCM(4)*qCM(3) + 45*qCM(4)*qpCM(4) - 
     2    7*qCM(3)*qpCM(4) - 42*(qpCM(4)**2) + 52*(qpPerp**2) + 13*tt) + 
     3    8*qCM(4)*(4*qCM(4) - qCM(3))* (qCM(4) + qpCM(4))*
     4    (qCM(4)*(qpPerp**2) + qpCM(4)*tt) + 
     5    qsq*(8*(4*qCM(4) - qCM(3))*qpCM(4)* 
     6    ((qCM(4)**2) - (qpCM(4)**2)) + 
     7    2*(42*(qCM(4)**2) - 3*qCM(3)*qpCM(4) + 
     8    qCM(4)*(-5*qCM(3) + 16*qpCM(4)))*(qpPerp**2) +
     9    (13*(qCM(4)**2) + 3*qCM(4)*(qCM(3) + 15*qpCM(4)) + 
     1    qpCM(4)*(-qCM(3) + 16*qpCM(4)))*tt)))/
     2    (16.*sqrt(2.)*qCM(3)*qsq*sqrt((qsq**2) - 4*s*tt))

       Ur(66)=-(sqrt(s)*((qsq**3) + (qsq**2)*
     1    ((qCM(4)**2) - qCM(4)*qCM(3) + qCM(4)*qpCM(4) - 
     2    3*qCM(3)*qpCM(4) - 2*(qpCM(4)**2) + 4*(qpPerp**2) + tt) - 
     3    8*qCM(4)*qCM(3)*(qCM(4) + qpCM(4))*
     4    (qCM(4)*(qpPerp**2) + qpCM(4)*tt) + qsq*(8*qCM(3)*qpCM(4)*
     5    (-(qCM(4)**2) + (qpCM(4)**2)) + 
     6    2*(2*(qCM(4)**2) - 9*qCM(4)*qCM(3) + qCM(3)*qpCM(4))*
     7    (qpPerp**2) + ((qCM(4)**2) - 5*qCM(3)*qpCM(4) + 
     8    qCM(4)*(-qCM(3) + qpCM(4)))*tt)))/
     9    (16.*sqrt(2.)*qCM(3)*qsq*sqrt((qsq**2) - 4*s*tt))

       Ur(67)=(qpPerp*sqrt(s)*((3*(qCM(4)**2) - 3*qCM(3)*qpCM(4) + 
     1    qCM(4)*(-qCM(3) + qpCM(4)))*tt + 
     2    qsq*(-((qCM(4) + qCM(3) - 2*qpCM(4))*
     3    (qCM(4) - qpCM(4))) + 4*tt)))/
     4    (4.*sqrt(2.)*qCM(3)*sqrt((qsq**2) - 4*s*tt))

       Ur(68)=(qpPerp*sqrt(s)*(((qCM(4)**2) -3*qCM(4)*(qCM(3)-qpCM(4)) - 
     1    qCM(3)*qpCM(4))*tt + qsq*(-3*(qCM(4) + qCM(3) - 2*qpCM(4))*
     2    (qCM(4) - qpCM(4)) + 4*tt)))/
     3    (4.*sqrt(2.)*qCM(3)*sqrt((qsq**2) - 4*s*tt))

       Ur(69)=(sqrt(s)*(-((qCM(4)*(qCM(3) - 2*qpCM(4)) + 
     1    (qpCM(4)**2))*(qsq**2)) + 2*qCM(4)*(qCM(4) - qCM(3))*
     2    (qCM(4) + qpCM(4))* (qCM(4)*(qpPerp**2) + qpCM(4)*tt) + 
     3    qsq*(2*(qCM(4) -qCM(3))*qpCM(4)* ((qCM(4)**2) -(qpCM(4)**2)) + 
     4    2*((qCM(4)**2) + qCM(3)*qpCM(4) + 
     5    qCM(4)*(-3*qCM(3) + qpCM(4)))*(qpPerp**2) + 
     6    (-(qCM(4)*qCM(3)) + 2*qCM(4)*qpCM(4) - 
     7    2*qCM(3)*qpCM(4) + (qpCM(4)**2))*tt)))/
     8    (2.*sqrt(2.)*qCM(3)*qsq*sqrt((qsq**2) - 4*s*tt))

       Ur(70)=(qpPerp*(3*(qCM(4) + qCM(3) - 2*qpCM(4))*qsq + 
     1    (3*qCM(4) - 4*qCM(3))*tt))/(8.*sqrt(2.)*qCM(3))

       Ur(71_=((13*qCM(4) + 3*qCM(3) - 10*qpCM(4))*(qsq**2) + 
     1    8*qCM(4)*(4*qCM(4) - qCM(3))*
     2    (qCM(4)*(qpPerp**2) + qpCM(4)*tt) + qsq*(8*(4*qCM(4) -qCM(3))*
     3    (qCM(4) - qpCM(4))*qpCM(4) + 2*(16*qCM(4) + 7*qCM(3))*
     4    (qpPerp**2) + (13*qCM(4) + 3*qCM(3) + 16*qpCM(4))*tt))/
     5    (32.*sqrt(2.)*qCM(3)*qsq)

       Ur(72)=((qCM(4) - qCM(3) - 2*qpCM(4))*(qsq**2) + 
     1    qsq*(2*qCM(3)*(4*qpCM(4)*(-qCM(4) +qpCM(4)) + 3*(qpPerp**2)) + 
     2   (qCM(4) - qCM(3))*tt) - 
     3   8*qCM(4)*qCM(3)*(qCM(4)*(qpPerp**2) + qpCM(4)*tt)
     4   )/(32.*sqrt(2.)*qCM(3)*qsq)

       Ur(73)=((qCM(3) - qpCM(4))*(qsq**2) + 
     1    qsq*(-2*(qCM(4) - qCM(3))*(qCM(4) - qpCM(4)\
     2    )*qpCM(4) - 2*qCM(4)*(qpPerp**2) + 
     3   (qCM(3) - qpCM(4))*tt) - 2*qCM(4)*(qCM(4) - qCM(3))*
     4   (qCM(4)*(qpPerp**2) + qpCM(4)*tt))/
     5   (4.*sqrt(2.)*qCM(3)*qsq)

       Ur(74)=(qpCM(4)*qpPerp*((qsq**2) + 
     1    2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt))/
     2    (16.*sqrt(2.)*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(75)=(((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt)*
     1    (3*(qsq**2) + qCM(4)*
     2    (8*qCM(4)*(qpPerp**2) + 3*(qCM(4) + qpCM(4))*tt) + 
     3    qsq*(8*(qpPerp**2) + 3*((qCM(4)**2) + qCM(4)*qpCM(4) - 
     4    2*(qpCM(4)**2) + tt))))/
     5    (16.*sqrt(2.)*qCM(3)*qsq*sqrt(s)*sqrt(-tt)*
     6    sqrt((qsq**2) - 4*s*tt))

       Ur(76)=-(((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt)*
     1    ((qsq**2) + qCM(4)*(qCM(4) + qpCM(4))*tt + 
     2    qsq*((qCM(4)**2) + qCM(4)*qpCM(4) - 2*(qpCM(4)**2) + tt)))/
     3    (16.*sqrt(2.)*qCM(3)*qsq*sqrt(s)*sqrt(-tt)*
     4    sqrt((qsq**2) - 4*s*tt))

       Ur(77)=-(qpPerp*((qCM(4) - qpCM(4))*qsq + qCM(4)*tt)*
     1    ((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt)
     2    )/(8.*sqrt(2.)*qsq*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(78)=(qCM(3)*(qpPerp**2)*((qsq**2) + 
     1    2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt))/
     2    (4.*sqrt(2.)*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(79)=-(sqrt(s)*((qsq**2) + qsq*((qCM(4)**2) + qCM(4)*qpCM(4) - 
     1    2*(qpCM(4)**2) + 4*(qpPerp**2) + tt) + 
     2    qCM(4)*(4*qCM(4)*(qpPerp**2) + (qCM(4) + qpCM(4))*tt)))/
     3    (8.*qCM(3)*qsq*sqrt((qsq**2) - 4*s*tt))

       Ur(80)=-(qpPerp*sqrt(s)*((qCM(4) - qpCM(4))*qsq + 
     1   (qCM(4) - 2*qpCM(4))*tt))/
     2   (2.*qsq*sqrt((qsq**2) - 4*s*tt))

       Ur(81)=(qpPerp*sqrt(s)*((qCM(4) - qpCM(4))*qsq + qCM(4)*tt))/
     1    (2.*qsq*sqrt((qsq**2) - 4*s*tt))

       Ur(82)=(sqrt(s)*((qsq**3) +(qsq**2)*
     1    ((qCM(4)**2) +5*qCM(4)*qpCM(4) - 
     2    4*(qpCM(4)**2) + 4*(qpPerp**2) + tt) + 
     3    4*(qCM(4)**2)*(qCM(4) + qpCM(4))*
     4    (qCM(4)*(qpPerp**2) + qpCM(4)*tt) + 
     5    qsq*(4*(qCM(4),3)*qpCM(4) - 4*qCM(4)*(qpCM(4),3) + 
     6    4*qCM(4)*(2*qCM(4) + qpCM(4))*(qpPerp**2) + 
     7    (qCM(4)**2)*tt + 5*qCM(4)*qpCM(4)*tt + 
     8    2*(qpCM(4)**2)*tt)))/
     9    (4.*qCM(3)*qsq*sqrt((qsq**2) - 4*s*tt))

       Ur(83)=(qpPerp*sqrt(s)*((qCM(4) - qpCM(4))*qsq + 
     1    (qCM(4) + qpCM(4))*tt))/
     2    (2.*qsq*sqrt((qsq**2) - 4*s*tt))

       Ur(84)=-((qsq**2) + qsq*(4*(qCM(4) - qpCM(4))*qpCM(4) - 
     1    2*(qpPerp**2) + tt) + 
     2    4*qCM(4)*(qCM(4)*(qpPerp**2) + qpCM(4)*tt))/
     3    (16.*qsq)

       Ur(85)=(qsq + 2*(qpPerp**2) + tt)/(16.*qsq)

       Ur(86)=(qpPerp*((qCM(4) - 2*qpCM(4))*qsq + qCM(4)*tt))/
     1    (4.*qCM(3)*qsq)

       Ur(87)=-(qpPerp*((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 
     1    4*s*tt)*(11*(qsq**2) + 4*qCM(4)*(qCM(4) + 4*qCM(3))*tt + 
     2    qsq*(12*(qCM(4)**2) + 16*qCM(4)*qCM(3) - 
     3    2*qCM(4)*qpCM(4) - 6*qCM(3)*qpCM(4) + 3*tt)))/
     4    (128.*qCM(3)*qsq*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(88)=-(qpPerp*((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 
     1    4*s*tt)*(7*(qsq**2) + 4*(qCM(4)**2)*tt - 
     2    qsq*(-2*(6*(qCM(4)**2) - 5*qCM(4)*qpCM(4) + 
     3    qCM(3)*qpCM(4)) + tt)))/
     4    (128.*qCM(3)*qsq*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(89)=(((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt)*
     1    ((qsq**2) + 2*qCM(4)*(3*qCM(4) + qCM(3))*
     2    (qpPerp**2) + (qCM(4) - qCM(3))*(qCM(4) + qpCM(4))*tt + 
     3    qsq*((qCM(4) - qpCM(4))* (qCM(4) - qCM(3) + 2*qpCM(4)) + 
     4    6*(qpPerp**2) + tt)))/
     5    (32.*qCM(3)*qsq*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(90)=-(((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 4*s*tt)*
     1    ((qsq**2) - 2*qCM(4)*(qCM(4) + 3*qCM(3))*
     2    (qpPerp**2) + (qCM(4) - qCM(3))* (qCM(4) + qpCM(4))*tt + 
     3    qsq*((qCM(4) - qpCM(4))* (qCM(4) - qCM(3) + 2*qpCM(4)) - 
     4    2*(qpPerp**2) + tt)))/
     5    (32.*qCM(3)*qsq*sqrt(s)*sqrt(-tt)*sqrt((qsq**2) - 4*s*tt))

       Ur(91)=(qpPerp*((qsq**2) + 2*(qCM(4) - qpCM(4))*qsq*sqrt(s) - 
     1    4*s*tt)*((qsq**2) + qCM(4)*(qCM(4) + qCM(3))*tt - 
     2    qsq*(-3*(qCM(4)**2) - qCM(4)*qCM(3) + 
     3    4*qCM(4)*qpCM(4) + 2*qCM(3)*qpCM(4) + tt)))/
     4    (16.*qCM(3)*qsq*sqrt(s)*sqrt(-t)*sqrt((qsq**2) - 4*s*tt))

       Ur(92)=(sqrt(s)*(13*(qsq**3) + (qsq**2)*
     1    (13*(qCM(4)**2) - 3*qCM(4)*qCM(3) + 45*qCM(4)*qpCM(4) + 
     2    7*qCM(3)*qpCM(4) - 42*(qpCM(4)**2) + 52*(qpPerp**2) + 13*tt) + 
     3    8*qCM(4)*(4*qCM(4) + qCM(3))* (qCM(4) + qpCM(4))*
     4    (qCM(4)*(qpPerp**2) + qpCM(4)*tt) + 
     5    qsq*(8*(4*qCM(4) + qCM(3))*qpCM(4)* 
     6    ((qCM(4)**2) - (qpCM(4)**2)) + 
     7    2*(42*(qCM(4)**2) + 3*qCM(3)*qpCM(4) + 
     8    qCM(4)*(5*qCM(3) + 16*qpCM(4)))*(qpPerp**2)\
     9    + (13*(qCM(4)**2) - 3*qCM(4)*(qCM(3) - 15*qpCM(4)) + 
     1    qpCM(4)*(qCM(3) + 16*qpCM(4)))*tt)))/
     2    (16.*sqrt(2.)*qCM(3)*(qsq**2)*sqrt((qsq**2) - 4*s*tt))

       Ur(93)=(sqrt(s)*((qsq**3) + (qsq**2)*
     1    ((qCM(4)**2) + qCM(4)*qCM(3) + qCM(4)*qpCM(4) + 
     2    3*qCM(3)*qpCM(4) - 2*(qpCM(4)**2) + 4*(qpPerp**2) + tt) + 
     3    8*qCM(4)*qCM(3)*(qCM(4) + qpCM(4))*
     4    (qCM(4)*(qpPerp**2) + qpCM(4)*tt) + qsq*(8*qCM(3)*qpCM(4)*
     5    ((qCM(4)**2) - (qpCM(4)**2)) + 2*(2*(qCM(4)**2) 
     6    + 9*qCM(4)*qCM(3) - qCM(3)*qpCM(4))*(qpPerp**2) + 
     7    ((qCM(4)**2) + 5*qCM(3)*qpCM(4) + 
     8    qCM(4)*(qCM(3) + qpCM(4)))*tt)))/
     9    (16.*sqrt(2.)*qCM(3)*(qsq**2)*sqrt((qsq**2) - 4*s*tt))

       Ur(94)=(qpPerp*sqrt(s)*((3*(qCM(4)**2) + 3*qCM(3)*qpCM(4) + 
     1    qCM(4)*(qCM(3) + qpCM(4)))*tt + 
     2    qsq*(-((qCM(4) - qCM(3) - 2*qpCM(4))*
     3    (qCM(4) - qpCM(4))) + 4*tt)))/
     4    (4.*sqrt(2.)*qCM(3)*qsq*sqrt((qsq**2) - 4*s*tt))

       Ur(95)=(qpPerp*sqrt(s)*(((qCM(4)**2) + qCM(3)*qpCM(4) + 
     1    3*qCM(4)*(qCM(3) + qpCM(4)))*tt + 
     2    qsq*(-3*(qCM(4) - qCM(3) - 2*qpCM(4))*
     3    (qCM(4) - qpCM(4)) + 4*tt)))/
     4    (4.*sqrt(2.)*qCM(3)*qsq*sqrt((qsq**2) - 4*s*tt))

       Ur(96)=-(sqrt(s)*((-(qpCM(4)**2) + qCM(4)*
     1    (qCM(3) + 2*qpCM(4)))*(qsq**2) + 2*qCM(4)*(qCM(4) + qCM(3))*
     2    (qCM(4) + qpCM(4))* (qCM(4)*(qpPerp**2) + qpCM(4)*tt) + 
     3    qsq*(2*(qCM(4) + qCM(3))*qpCM(4)*
     4    ((qCM(4)**2) - (qpCM(4)**2)) + 
     5    2*((qCM(4)**2) - qCM(3)*qpCM(4) + 
     6    qCM(4)*(3*qCM(3) + qpCM(4)))*(qpPerp**2) + 
     7    (qpCM(4)*(2*qCM(3) + qpCM(4)) + 
     8    qCM(4)*(qCM(3) + 2*qpCM(4)))*tt)))/
     9    (2.*sqrt(2.)*qCM(3)*(qsq**2)*sqrt((qsq**2) - 4*s*tt))

       Ur(97)=(qpPerp*((-3*qCM(4) + 3*qCM(3) + 6*qpCM(4))*qsq - 
     1    (3*qCM(4) + 4*qCM(3))*tt))/(8.*sqrt(2.)*qCM(3)*qsq)

       Ur(98)=((13*qCM(4) - 3*qCM(3) - 10*qpCM(4))*(qsq**2) + 
     1    8*qCM(4)*(4*qCM(4) +qCM(3))*(qCM(4)*(qpPerp**2) +qpCM(4)*tt) + 
     2    qsq*(8*(4*qCM(4) + qCM(3))* (qCM(4) - qpCM(4))*qpCM(4) + 
     3    2*(16*qCM(4) - 7*qCM(3))*(qpPerp**2) + 
     4   (13*qCM(4) - 3*qCM(3) + 16*qpCM(4))*tt))/
     5   (32.*sqrt(2.)*qCM(3)*(qsq**2))

       Ur(99)=((qCM(4) + qCM(3) - 2*qpCM(4))*(qsq**2) + 
     1    qsq*(-2*qCM(3)*
     2    (4*qpCM(4)*(-qCM(4) + qpCM(4)) + 3*(qpPerp**2)) + 
     3    (qCM(4) + qCM(3))*tt) + 
     4    8*qCM(4)*qCM(3)*(qCM(4)*(qpPerp**2) + qpCM(4)*tt))
     5    /(32.*sqrt(2.)*qCM(3)*(qsq**2))

       Ur(100)=-((qCM(3) + qpCM(4))*(qsq**2) + 
     1    2*qCM(4)*(qCM(4) + qCM(3))* (qCM(4)*(qpPerp**2) +qpCM(4)*tt) + 
     2    qsq*(2*(qCM(4) + qCM(3))*(qCM(4) - qpCM(4)\
     3    )*qpCM(4) + 2*qCM(4)*(qpPerp**2) + (qCM(3) + qpCM(4))*tt))/
     4    (4.*sqrt(2.)*qCM(3)*(qsq**2))
	
      endif

c iexact=0:
c All quantities are evaluated at leading order in the 1/Q expansion 
c (see Belitsky, Mueller and Kirchner, Nucl.Phys. B629 (2002) 323-392,
c ArXiv:hep-ph/0112108v2)

      if (iexact.eq.0) then

       Ur(1)=(-13*(pqPerp**2)*sqrt(((pqPerp**2) + (m_p**2)*(xB**2))/
     1    (1 - xB)))/16.
       Ur(2)=sqrt(((pqPerp**2) + (m_p**2)*(xB**2))/(1 - xB))/16.
       Ur(3)=(-3*(pqPerp**2)*sqrt(((pqPerp**2) + (m_p**2)*(xB**2))/
     1    (1 - xB)))/16.
       Ur(4)=(7*sqrt(((pqPerp**4) + (m_p**2)*(pqPerp**2)*(xB**2))/
     1    (1 - xB)))/16.
       Ur(5)=(3*sqrt(-((-1 + xB)*((pqPerp**4) + 
     1    (m_p**2)*(pqPerp**2)*(xB**2)))))/(16.*xB)
       Ur(6)=(3*sqrt(((pqPerp**4) + (m_p**2)*(pqPerp**2)*(xB**2))/
     1    (1 - xB)))/16.
       Ur(7)=-sqrt(((pqPerp**2) + (m_p**2)*(xB**2))/(1 - xB))/8.
       Ur(8)=-sqrt(((pqPerp**4) + (m_p**2)*(pqPerp**2)*(xB**2))/
     1    (1 - xB))/2.
       Ur(9)=((pqPerp**3)*(13 - 6*xB) + 7*(m_p**2)*qpPerp*(xB**2))/
     1    (8.*sqrt(2.)*(-1 + xB))
       Ur(10)=qpPerp/(4.*sqrt(2.))
       Ur(11)=((pqPerp**3)*(3 - 2*xB) + (m_p**2)*qpPerp*(xB**2))/
     1    (8.*sqrt(2.)*(-1 + xB))
       Ur(12)=(-5*(m_p**2)*(xB**2) + (pqPerp**2)*(-11 + 6*xB))/
     1    (16.*sqrt(2.)*(-1 + xB))
       Ur(13)=-3/(16.*sqrt(2.))
       Ur(14)=(-((m_p**2)*(xB**2)) + (pqPerp**2)*(-7 + 6*xB))/
     1    (16.*sqrt(2.)*(-1 + xB))
       Ur(15)=-qpPerp/(4.*sqrt(2.))
       Ur(16)=((pqPerp**2)*(4 - 3*xB) + (m_p**2)*(xB**2))/
     1    (4.*sqrt(2.)*(-1 + xB))
       Ur(17)=-qpPerp/(4.*sqrt(2.))
       Ur(18)=((pqPerp**2)*(1 - 6*xB) - 5*(m_p**2)*(xB**2))/
     1    (16.*sqrt(2.)*(-1 + xB))
       Ur(19)=3/(16.*sqrt(2.))
       Ur(20)=((pqPerp**2)*(5 - 6*xB) - (m_p**2)*(xB**2))/
     1    (16.*sqrt(2.)*(-1 + xB))
       Ur(21)=-(qpPerp*(2*(m_p**2)*(xB**2) + (pqPerp**2)*(1 + xB)))/
     1    (4.*sqrt(2.)*(-1 + xB))
       Ur(22)=qpPerp/(4.*sqrt(2.))
       Ur(23)=((pqPerp**3)*(1 - 3*xB) - 2*(m_p**2)*qpPerp*(xB**2))/
     1    (4.*sqrt(2.)*(-1 + xB))
       Ur(24)=((m_p**2)*(xB**2) + (pqPerp**2)*(-2 + 3*xB))/
     1    (4.*sqrt(2.)*(-1 + xB))
       Ur(25)=(3*sqrt(((pqPerp**4) + (m_p**2)*(pqPerp**2)*(xB**2))/
     1    (2 - 2*xB)))/8.
       Ur(26)=-sqrt(-((-1 + xB)*((pqPerp**4) + 
     1    (m_p**2)*(pqPerp**2)*(xB**2))))/(8.*sqrt(2.)*xB)
       Ur(27)=(-3*sqrt(((pqPerp**2) + (m_p**2)*(xB**2))/(2 - 2*xB)))/8.
       Ur(28)=sqrt(((pqPerp**2) + (m_p**2)*(xB**2))/(2 - 2*xB))/8.
       Ur(29)=((pqPerp**2)*(-1 + 2*xB)*sqrt(((pqPerp**2) + 
     1    (m_p**2)*(xB**2))/(2 - 2*xB)))/(2.*xB)
       Ur(30)=-((pqPerp**2)*(7 -6*xB) + (m_p**2)*(xB**2))/
     1    (16.*(-1 + xB))
       Ur(31)=0.0625
       Ur(32)=-qpPerp/4.
       Ur(33)=qpPerp/4.
       Ur(34)=qpPerp/4.
       Ur(35)=-0.0625
       Ur(36)=qpPerp/4.
       Ur(37)=-qpPerp/4.
       Ur(38)=((pqPerp**2) + (m_p**2)*(xB**2))/(-8 + 8*xB)
       Ur(39)=-qpPerp/4.
       Ur(40)=-sqrt(((pqPerp**2) + (m_p**2)*(xB**2))/(1 - xB))/16.
       Ur(41)=(qpPerp*(5 - 9*xB + 4*(xB**2))*
     1   sqrt(-(((pqPerp**2) + (m_p**2)*(xB**2))/((-1+xB)**3)))/
     1   (16.*xB)
       Ur(42)=(3*(pqPerp**3)*sqrt(((pqPerp**2) + (m_p**2)*(xB**2))/
     1   (1 - xB)))/16.
       Ur(43)=((1 - 4*xB)*sqrt(((pqPerp**4) + 
     1   (m_p**2)*(pqPerp**2)*(xB**2))/(1 - xB)))/(16.*xB)
       Ur(44)=((pqPerp**2)*sqrt(((pqPerp**2) + (m_p**2)*(xB**2))/
     1   (1 - xB)))/8.
       Ur(45)=sqrt(((pqPerp**4) + (m_p**2)*(pqPerp**2)*(xB**2))/
     1   (1 - xB))/(4.*xB)
       Ur(46)=(3*qpPerp)/(4.*sqrt(2.))
       Ur(47)=-5/(16.*sqrt(2.))
       Ur(48)=(3*(pqPerp**4)*(-2 + xB) - 
     1   3*(m_p**2)*(pqPerp**2)*(xB**2))/(16.*sqrt(2.)*(-1 + xB))
       Ur(49)=-1/(16.*sqrt(2.))
       Ur(50)=(pqPerp**3)/(4.*sqrt(2.))
       Ur(51)=-1/(4.*sqrt(2.))
       Ur(52)=((pqPerp**3)*(1 - 2*xB) - (m_p**2)*qpPerp*(xB**2))/
     1   (8.*sqrt(2.)*(-1 + xB))
       Ur(53)=5/(16.*sqrt(2.))
       Ur(54)=(3*xB*((pqPerp**4) + (m_p**2)*(pqPerp**2)*xB))/
     1   (16.*sqrt(2.)*(-1 + xB))
       Ur(55)=1/(16.*sqrt(2.))
       Ur(56)=-qpPerp/(4.*sqrt(2.))
       Ur(57)=(pqPerp**3)/(4.*sqrt(2.))
       Ur(58)=(-3*qpPerp)/(4.*sqrt(2.))
       Ur(59)=1/(4.*sqrt(2.))
       Ur(60)=(-7*sqrt(((pqPerp**4) + (m_p**2)*(pqPerp**2)*(xB**2))/
     1   (1 - xB)))/16.
       Ur(61)=(-3*sqrt(((pqPerp**4) + (m_p**2)*(pqPerp**2)*(xB**2))/
     1   (1 - xB)))/16.
       Ur(62)=(-3*(pqPerp**2)*sqrt(((pqPerp**2) + (m_p**2)*(xB**2))/
     1   (1 - xB)))/8.
       Ur(63)=(-5*(pqPerp**2)*sqrt(((pqPerp**2) + (m_p**2)*(xB**2))/
     1   (1 - xB)))/8.
       Ur(64)=sqrt(((pqPerp**4) + (m_p**2)*(pqPerp**2)*(xB**2))/
     1   (1 - xB))/2.
       Ur(65)=((pqPerp**2)*(11 - 6*xB) + 5*(m_p**2)*(xB**2))/
     1   (16.*sqrt(2.)*(-1 + xB))
       Ur(66)=((pqPerp**2)*(7 - 6*xB) + (m_p**2)*(xB**2))/
     1   (16.*sqrt(2.)*(-1 + xB))
       Ur(67)=(-((pqPerp**3)*(-3 + xB)) + 2*(m_p**2)*qpPerp*(xB**2))/
     1   (4.*sqrt(2.)*(-1 + xB))
       Ur(68)=((pqPerp**3)*(5 - 3*xB) + 2*(m_p**2)*qpPerp*(xB**2))/
     1   (4.*sqrt(2.)*(-1 + xB))
       Ur(69)=(-((m_p**2)*(xB**2)) + (pqPerp**2)*(-4 + 3*xB))/
     1   (4.*sqrt(2.)*(-1 + xB))
       Ur(70)=-(qpPerp*(7*(m_p**2)*(xB**2) + (pqPerp**2)*(1 + 6*xB)))/
     1   (8.*sqrt(2.)*(-1 + xB))
       Ur(71)=(5*(m_p**2)*(xB**2) + (pqPerp**2)*(-1 + 6*xB))/
     1   (16.*sqrt(2.)*(-1 + xB))
       Ur(72)=((m_p**2)*(xB**2) + (pqPerp**2)*(-5 + 6*xB))/
     1   (16.*sqrt(2.)*(-1 + xB))
       Ur(73)=((pqPerp**2)*(2 - 3*xB) - (m_p**2)*(xB**2))/
     1   (4.*sqrt(2.)*(-1 + xB))
       Ur(74)=sqrt(-((-1 + xB)*((pqPerp**4) + 
     1   (m_p**2)*(pqPerp**2)*(xB**2))))/(8.*sqrt(2.)*xB)
       Ur(75)=(3*sqrt(((pqPerp**2) + (m_p**2)*(xB**2))/(2 - 2*xB)))/8.
       Ur(76)=-sqrt(((pqPerp**2) + (m_p**2)*(xB**2))/(2 - 2*xB))/8.
       Ur(77)=sqrt(((pqPerp**4) + (m_p**2)*(pqPerp**2)*(xB**2))/
     1   (2 - 2*xB))/4.
       Ur(78)=((pqPerp**2)*sqrt(((pqPerp**2) + (m_p**2)*(xB**2))/
     1   (2 - 2*xB)))/(2.*xB)
       Ur(79)=-0.0625
       Ur(80)=qpPerp/4.
       Ur(81)=-qpPerp/4.
       Ur(82)=((pqPerp**2) + (m_p**2)*(xB**2))/(8 - 8*xB)
       Ur(83)=-qpPerp/4.
       Ur(84)=((m_p**2)*(xB**2) + (pqPerp**2)*(-5 + 6*xB))/
     1   (16.*(-1 + xB))
       Ur(85)=0.0625
       Ur(86)=-qpPerp/4.
       Ur(87)=-(qpPerp*(5 - 11*xB + 6*(xB**2))*
     1   sqrt(-(((pqPerp**2) + (m_p**2)*(xB**2))/((-1 + xB)**3))))/
     2   (16.*xB)
       Ur(88)=-((1 + 2*xB)*sqrt(((pqPerp**4) + 
     1   (m_p**2)*(pqPerp**2)*(xB**2))/(1 - xB)))/(16.*xB)
       Ur(89)=sqrt(((pqPerp**2) + (m_p**2)*(xB**2))/(1 - xB))/8.
       Ur(90)=-sqrt(((pqPerp**2) + (m_p**2)*(xB**2))/(1 - xB))/8.
       Ur(91)=-(qpPerp*(1 - 3*xB + 2*(xB**2))*
     1    sqrt(-(((pqPerp**2) + (m_p**2)*(xB**2))/((-1 + xB)**3))))/(4.*xB)
       Ur(92)=5/(16.*sqrt(2.))
       Ur(93)=1/(16.*sqrt(2.))
       Ur(94)=-qpPerp/(4.*sqrt(2.))
       Ur(95)=(-3*qpPerp)/(4.*sqrt(2.))
       Ur(96)=1/(4.*sqrt(2.))
       Ur(97)=(3*qpPerp)/(4.*sqrt(2.))
       Ur(98)=-5/(16.*sqrt(2.))
       Ur(99)=-1/(16.*sqrt(2.))
       Ur(100)=-1/(4.*sqrt(2.))

      endif
     
c---------------------------- VCS cross sections -------------------------------*
c need to add +1 to all indices, since fortran starts at 1, but c++ starts at 0.

      SigmaVCSPol0(1)=(3*((IMvcs(1,1))**2) - 2*((IMvcs(1,2))**2) + 
     1   3*((IMvcs(1,3))**2) + 3*((IMvcs(2,1))**2) - 
     2   2*((IMvcs(2,2))**2) + 3*((IMvcs(2,3))**2) + 
     3   3*((IMvcs(3,1))**2) - 2*((IMvcs(3,2))**2) + 
     4   3*((IMvcs(3,3))**2) + 3*((IMvcs(4,1))**2) - 
     5   2*((IMvcs(4,2))**2) + 3*((IMvcs(4,3))**2) + 
     6   3*((RMvcs(1,1))**2) - 2*((RMvcs(1,2))**2) + 
     7   3*((RMvcs(1,3))**2) + 3*((RMvcs(2,1))**2) - 
     8   2*((RMvcs(2,2))**2) + 3*((RMvcs(2,3))**2) + 
     9   3*((RMvcs(3,1))**2) - 2*((RMvcs(3,2))**2) + 
     1   3*((RMvcs(3,3))**2) + 3*((RMvcs(4,1))**2) - 
     2   2*((RMvcs(4,2))**2) + 3*((RMvcs(4,3))**2))/4.

      SigmaVCSPol0(2)=(((IMvcs(1,1))**2) + 2*((IMvcs(1,2))**2) + 
     1   ((IMvcs(1,3))**2) + ((IMvcs(2,1))**2) + 
     2   2*((IMvcs(2,2))**2) + ((IMvcs(2,3))**2) + 
     3   ((IMvcs(3,1))**2) + 2*((IMvcs(3,2))**2) + 
     4   ((IMvcs(3,3))**2) + ((IMvcs(4,1))**2) + 
     5   2*((IMvcs(4,2))**2) + ((IMvcs(4,3))**2) + 
     6   ((RMvcs(1,1))**2) + 2*((RMvcs(1,2))**2) + 
     7   ((RMvcs(1,3))**2) + ((RMvcs(2,1))**2) + 
     8   2*((RMvcs(2,2))**2) + ((RMvcs(2,3))**2) + 
     9   ((RMvcs(3,1))**2) + 2*((RMvcs(3,2))**2) + 
     1   ((RMvcs(3,3))**2) + ((RMvcs(4,1))**2) + 
     2   2*((RMvcs(4,2))**2) + ((RMvcs(4,3))**2))/4.

      SigmaVCSPol0(3)=(IMvcs(1,2)*(IMvcs(1,1) - IMvcs(1,3)) + 
     1   IMvcs(2,2)*(IMvcs(2,1) - IMvcs(2,3)) + 
     2   IMvcs(3,2)*(IMvcs(3,1) - IMvcs(3,3)) + 
     3   IMvcs(4,2)*(IMvcs(4,1) - IMvcs(4,3)) + 
     4   RMvcs(1,2)*(RMvcs(1,1) - RMvcs(1,3)) + 
     5   RMvcs(2,2)*(RMvcs(2,1) - RMvcs(2,3)) + 
     6   RMvcs(3,2)*(RMvcs(3,1) - RMvcs(3,3)) + 
     7   RMvcs(4,2)*(RMvcs(4,1) - RMvcs(4,3)))/sqrt(2.)

      SigmaVCSPol0(4)=(IMvcs(1,1)*IMvcs(1,3) + IMvcs(2,1)*IMvcs(2,3) + 
     1   IMvcs(3,1)*IMvcs(3,3) + IMvcs(4,1)*IMvcs(4,3) + 
     2   RMvcs(1,1)*RMvcs(1,3) + RMvcs(2,1)*RMvcs(2,3) + 
     3   RMvcs(3,1)*RMvcs(3,3) + RMvcs(4,1)*RMvcs(4,3))/2.

      SigmaVCSPol0(5)=sqrt(2.)*((IMvcs(1,1) - IMvcs(1,3))*RMvcs(1,2) + 
     1   IMvcs(1,2)*(-RMvcs(1,1) + RMvcs(1,3)) + 
     2   (IMvcs(2,1) - IMvcs(2,3))*RMvcs(2,2) + 
     3   IMvcs(2,2)*(-RMvcs(2,1) + RMvcs(2,3)) + 
     4   (IMvcs(3,1) - IMvcs(3,3))*RMvcs(3,2) + 
     5   IMvcs(3,2)*(-RMvcs(3,1) + RMvcs(3,3)) + 
     6   (IMvcs(4,1) - IMvcs(4,3))*RMvcs(4,2) + 
     7   IMvcs(4,2)*(-RMvcs(4,1) + RMvcs(4,3)))

      SigmaVCSPolX(1)=2*(IMvcs(1,1)*IMvcs(2,1) - IMvcs(1,3)*IMvcs(2,3) - 
     1   IMvcs(3,1)*IMvcs(4,1) + IMvcs(3,3)*IMvcs(4,3) + 
     2   RMvcs(1,1)*RMvcs(2,1) - RMvcs(1,3)*RMvcs(2,3) - 
     3   RMvcs(3,1)*RMvcs(4,1) + RMvcs(3,3)*RMvcs(4,3))

      SigmaVCSPolX(2)=sqrt(2.)*((IMvcs(1,1) + IMvcs(1,3))*IMvcs(2,2) + 
     1   IMvcs(1,2)*(IMvcs(2,1) + IMvcs(2,3)) - 
     2   (IMvcs(3,1) + IMvcs(3,3))*IMvcs(4,2) - 
     3   IMvcs(3,2)*(IMvcs(4,1) + IMvcs(4,3)) + 
     4   (RMvcs(1,1) + RMvcs(1,3))*RMvcs(2,2) + 
     5   RMvcs(1,2)*(RMvcs(2,1) + RMvcs(2,3)) - 
     6   (RMvcs(3,1) + RMvcs(3,3))*RMvcs(4,2) - 
     7   RMvcs(3,2)*(RMvcs(4,1) + RMvcs(4,3)))

      SigmaVCSPolX(3)=((IMvcs(2,1) + IMvcs(2,3))*RMvcs(1,2) - 
     1   IMvcs(2,2)*(RMvcs(1,1) + RMvcs(1,3)) + 
     2   (IMvcs(1,1) + IMvcs(1,3))*RMvcs(2,2) - 
     3   IMvcs(1,2)*(RMvcs(2,1) + RMvcs(2,3)) - 
     4   (IMvcs(4,1) + IMvcs(4,3))*RMvcs(3,2) + 
     5   IMvcs(4,2)*(RMvcs(3,1) + RMvcs(3,3)) - 
     6   (IMvcs(3,1) + IMvcs(3,3))*RMvcs(4,2) + 
     7   IMvcs(3,2)*(RMvcs(4,1) + RMvcs(4,3)))/sqrt(2.)

      SigmaVCSPolX(4)=(-(IMvcs(2,3)*RMvcs(1,1)) + IMvcs(2,1)*RMvcs(1,3) - 
     1   IMvcs(1,3)*RMvcs(2,1) + IMvcs(1,1)*RMvcs(2,3) + 
     2   IMvcs(4,3)*RMvcs(3,1) - IMvcs(4,1)*RMvcs(3,3) + 
     3   IMvcs(3,3)*RMvcs(4,1) - IMvcs(3,1)*RMvcs(4,3))/2.

      SigmaVCSPolY(2)=sqrt(2.)*((IMvcs(1,1) - IMvcs(1,3))*IMvcs(3,2) + 
     1   IMvcs(1,2)*(-IMvcs(3,1) + IMvcs(3,3)) + 
     2   (IMvcs(2,1) - IMvcs(2,3))*IMvcs(4,2) + 
     3   IMvcs(2,2)*(-IMvcs(4,1) + IMvcs(4,3)) + 
     4   (RMvcs(1,1) - RMvcs(1,3))*RMvcs(3,2) + 
     5   RMvcs(1,2)*(-RMvcs(3,1) + RMvcs(3,3)) + 
     6   (RMvcs(2,1) - RMvcs(2,3))*RMvcs(4,2) + 
     7   RMvcs(2,2)*(-RMvcs(4,1) + RMvcs(4,3)))

      SigmaVCSPolY(3)=(3*IMvcs(3,1)*RMvcs(1,1) - 2*IMvcs(3,2)*RMvcs(1,2) + 
     1   3*IMvcs(3,3)*RMvcs(1,3) + 3*IMvcs(4,1)*RMvcs(2,1) - 
     2   2*IMvcs(4,2)*RMvcs(2,2) + 3*IMvcs(4,3)*RMvcs(2,3) - 
     3   3*IMvcs(1,1)*RMvcs(3,1) + 2*IMvcs(1,2)*RMvcs(3,2) - 
     4   3*IMvcs(1,3)*RMvcs(3,3) - 3*IMvcs(2,1)*RMvcs(4,1) + 
     5   2*IMvcs(2,2)*RMvcs(4,2) - 3*IMvcs(2,3)*RMvcs(4,3))/2.

      SigmaVCSPolY(4)=(IMvcs(3,1)*RMvcs(1,1) + 2*IMvcs(3,2)*RMvcs(1,2) + 
     1   IMvcs(3,3)*RMvcs(1,3) + IMvcs(4,1)*RMvcs(2,1) + 
     2   2*IMvcs(4,2)*RMvcs(2,2) + IMvcs(4,3)*RMvcs(2,3) - 
     3   IMvcs(1,1)*RMvcs(3,1) - 2*IMvcs(1,2)*RMvcs(3,2) - 
     4   IMvcs(1,3)*RMvcs(3,3) - IMvcs(2,1)*RMvcs(4,1) - 
     5   2*IMvcs(2,2)*RMvcs(4,2) - IMvcs(2,3)*RMvcs(4,3))/2.

      SigmaVCSPolY(4)=((IMvcs(3,1) - IMvcs(3,3))*RMvcs(1,2) + 
     1   IMvcs(3,2)*(RMvcs(1,1) - RMvcs(1,3)) + 
     2   (IMvcs(4,1) - IMvcs(4,3))*RMvcs(2,2) + 
     3   IMvcs(4,2)*(RMvcs(2,1) - RMvcs(2,3)) + 
     4   (-IMvcs(1,1) + IMvcs(1,3))*RMvcs(3,2) + 
     5   IMvcs(1,2)*(-RMvcs(3,1) + RMvcs(3,3)) + 
     6   (-IMvcs(2,1) + IMvcs(2,3))*RMvcs(4,2) + 
     7   IMvcs(2,2)*(-RMvcs(4,1) + RMvcs(4,3)))/sqrt(2.)

      SigmaVCSPolY(5)=(IMvcs(3,3)*RMvcs(1,1) + IMvcs(3,1)*RMvcs(1,3) + 
     1   IMvcs(4,3)*RMvcs(2,1) + IMvcs(4,1)*RMvcs(2,3) - 
     2   IMvcs(1,3)*RMvcs(3,1) - IMvcs(1,1)*RMvcs(3,3) - 
     3   IMvcs(2,3)*RMvcs(4,1) - IMvcs(2,1)*RMvcs(4,3))/2.

      SigmaVCSPolZ(1)=2*(IMvcs(2,1)*IMvcs(3,1) - IMvcs(2,3)*IMvcs(3,3) + 
     1   IMvcs(1,1)*IMvcs(4,1) - IMvcs(1,3)*IMvcs(4,3) + 
     2   RMvcs(2,1)*RMvcs(3,1) - RMvcs(2,3)*RMvcs(3,3) + 
     3   RMvcs(1,1)*RMvcs(4,1) - RMvcs(1,3)*RMvcs(4,3))

      SigmaVCSPolZ(2)=sqrt(2.)*((IMvcs(2,1) + IMvcs(2,3))*IMvcs(3,2) + 
     1   IMvcs(2,2)*(IMvcs(3,1) + IMvcs(3,3)) + 
     2   (IMvcs(1,1) + IMvcs(1,3))*IMvcs(4,2) + 
     3   IMvcs(1,2)*(IMvcs(4,1) + IMvcs(4,3)) + 
     4   (RMvcs(2,1) + RMvcs(2,3))*RMvcs(3,2) + 
     5   RMvcs(2,2)*(RMvcs(3,1) + RMvcs(3,3)) + 
     6   (RMvcs(1,1) + RMvcs(1,3))*RMvcs(4,2) + 
     7   RMvcs(1,2)*(RMvcs(4,1) + RMvcs(4,3)))

      SigmaVCSPolZ(3)=((IMvcs(4,1) + IMvcs(4,3))*RMvcs(1,2) - 
     1   IMvcs(4,2)*(RMvcs(1,1) + RMvcs(1,3)) + 
     2   (IMvcs(3,1) + IMvcs(3,3))*RMvcs(2,2) - 
     3   IMvcs(3,2)*(RMvcs(2,1) + RMvcs(2,3)) + 
     4   (IMvcs(2,1) + IMvcs(2,3))*RMvcs(3,2) - 
     5   IMvcs(2,2)*(RMvcs(3,1) + RMvcs(3,3)) + 
     6   (IMvcs(1,1) + IMvcs(1,3))*RMvcs(4,2) - 
     7   IMvcs(1,2)*(RMvcs(4,1) + RMvcs(4,3)))/sqrt(2.)

      SigmaVCSPolZ(4)=(-(IMvcs(4,3)*RMvcs(1,1)) + IMvcs(4,1)*RMvcs(1,3) - 
     1   IMvcs(3,3)*RMvcs(2,1) + IMvcs(3,1)*RMvcs(2,3) - 
     2   IMvcs(2,3)*RMvcs(3,1) + IMvcs(2,1)*RMvcs(3,3) - 
     3   IMvcs(1,3)*RMvcs(4,1) + IMvcs(1,1)*RMvcs(4,3))/2.
          
c----------------------- Interference cross sections ---------------------------*
c need to add +1 to all indices, since fortran starts at 1, but c++ starts at 0.

      SigmaIPol0(1)=Jem(1,2)*(-2*RMvcs(1,1)*Ur(1)*qsq + 
     1   2*RMvcs(1,2)*Ur(25)*(Q**3) - 2*RMvcs(1,3)*Ur(40)*(Q**4)) + 
     2   Jem(3,2)*(-2*RMvcs(3,1)*Ur(1)*qsq + 
     3   2*RMvcs(3,2)*Ur(25)*(Q**3) - 2*RMvcs(3,3)*Ur(40)*(Q**4)) + 
     4   Jem(1,3)*(-2*RMvcs(1,1)*Ur(9)*qsq + 
     5   2*RMvcs(1,2)*Ur(30)*(Q**3) - 2*RMvcs(1,3)*Ur(46)*(Q**4)) + 
     6   Jem(3,3)*(-2*RMvcs(3,1)*Ur(9)*qsq + 
     7   2*RMvcs(3,2)*Ur(30)*(Q**3) - 2*RMvcs(3,3)*Ur(46)*(Q**4)) + 
     8   Jem(2,3)*(-2*RMvcs(2,1)*Ur(70)*qsq + 
     9   2*RMvcs(2,2)*Ur(84)*(Q**3) - 2*RMvcs(2,3)*Ur(97)*(Q**4)) + 
     1   Jem(4,3)*(-2*RMvcs(4,1)*Ur(70)*qsq + 
     2   2*RMvcs(4,2)*Ur(84)*(Q**3) - 2*RMvcs(4,3)*Ur(97)*(Q**4))

      SigmaIPol0(2)=Jem(1,2)*(-2*RMvcs(1,1)*Ur(3)*qsq - 
     1   2*RMvcs(1,2)*Ur(25)*(Q**3) - 2*RMvcs(1,3)*Ur(2)*(Q**4)) + 
     2   Jem(3,2)*(-2*RMvcs(3,1)*Ur(3)*qsq - 
     3   2*RMvcs(3,2)*Ur(25)*(Q**3)) - 2*RMvcs(3,3)*Ur(2)*(Q**4)) + 
     4   Jem(2,3)*(-2*RMvcs(2,1)*Ur(52)*qsq - 
     5   2*RMvcs(2,2)*Ur(84)*(Q**3) + 2*RMvcs(2,3)*Ur(17)*(Q**4)) + 
     6   Jem(4,3)*(-2*RMvcs(4,1)*Ur(52)*qsq - 
     7   2*RMvcs(4,2)*Ur(84)*(Q**3) + 2*RMvcs(4,3)*Ur(17)*(Q**4)) + 
     8   Jem(1,3)*(-2*RMvcs(1,1)*Ur(11)*qsq - 
     9   2*RMvcs(1,2)*Ur(30)*(Q**3) - 2*RMvcs(1,3)*Ur(10)*(Q**4)) + 
     1   Jem(3,3)*(-2*RMvcs(3,1)*Ur(11)*qsq - 
     2   2*RMvcs(3,2)*Ur(30)*(Q**3) - 2*RMvcs(3,3)*Ur(10)*(Q**4))

      SigmaIPol0(3)=Jem(1,2)*(-2*RMvcs(1,1)*Ur(4)*(Q**3) - 
     1   2*RMvcs(1,3)*Ur(41)*(Q**3) + 2*RMvcs(1,2)*Ur(27)*(Q**4)) + 
     2   Jem(3,2)*(-2*RMvcs(3,1)*Ur(4)*(Q**3) - 
     3   2*RMvcs(3,3)*Ur(41)*(Q**3) + 2*RMvcs(3,2)*Ur(27)*(Q**4)) + 
     4   Jem(1,3)*(-2*RMvcs(1,1)*Ur(12)*(Q**3) + 
     5   2*RMvcs(1,2)*Ur(32)*(Q**4) - 2*RMvcs(1,3)*Ur(47)*(Q**5)) + 
     6   Jem(3,3)*(-2*RMvcs(3,1)*Ur(12)*(Q**3) + 
     7   2*RMvcs(3,2)*Ur(32)*(Q**4) - 2*RMvcs(3,3)*Ur(47)*(Q**5)) + 
     8   Jem(2,3)*(-2*RMvcs(2,1)*Ur(71)*(Q**3) + 
     9   2*RMvcs(2,2)*Ur(86)*(Q**4) - 2*RMvcs(2,3)*Ur(98)*(Q**5)) + 
     1   Jem(4,3)*(-2*RMvcs(4,1)*Ur(71)*(Q**3) + 
     2   2*RMvcs(4,2)*Ur(86)*(Q**4) - 2*RMvcs(4,3)*Ur(98)*(Q**5))

      SigmaIPol0(4)=Jem(1,2)*(-2*RMvcs(1,3)*Ur(43)*(Q**3) - 
     1   2*RMvcs(1,1)*Ur(6)*(Q**3) + 2*RMvcs(1,2)*Ur(28)*(Q**4) + 
     2   Jem(3,2)*(-2*RMvcs(3,3)*Ur(43)*(Q**3) - 
     3   2*RMvcs(3,1)*Ur(6)*(Q**3) + 2*RMvcs(3,2)*Ur(28)*(Q**4)) + 
     4   Jem(1,3)*(-2*RMvcs(1,1)*Ur(14)*(Q**3) + 
     5   2*RMvcs(1,2)*Ur(33)*(Q**4) - 2*RMvcs(1,3)*Ur(49)*(Q**5)) + 
     6   Jem(3,3)*(-2*RMvcs(3,1)*Ur(14)*(Q**3) + 
     7   2*RMvcs(3,2)*Ur(33)*(Q**4) - 2*RMvcs(3,3)*Ur(49)*(Q**5)) + 
     8   Jem(2,3)*(-2*RMvcs(2,1)*Ur(72)*(Q**3) - 
     9   2*RMvcs(2,2)*Ur(86)*(Q**4) - 2*RMvcs(2,3)*Ur(99)*(Q**5)) + 
     1   Jem(4,3)*(-2*RMvcs(4,1)*Ur(72)*(Q**3) - 
     2   2*RMvcs(4,2)*Ur(86)*(Q**4) - 2*RMvcs(4,3)*Ur(99)*(Q**5))

      SigmaIPol0(5)=Jem(1,2)*(-2*RMvcs(1,3)*Ur(3)*qsq + 
     1   2*RMvcs(1,2)*Ur(26)*(Q**3) - 2*RMvcs(1,1)*Ur(2)*(Q**4)) + 
     2   Jem(3,2)*(-2*RMvcs(3,3)*Ur(3)*qsq + 
     3   2*RMvcs(3,2)*Ur(26)*(Q**3) - 2*RMvcs(3,1)*Ur(2)*(Q**4)) + 
     4   Jem(1,3)*(-2*RMvcs(1,3)*Ur(11)*qsq - 
     5   2*RMvcs(1,1)*Ur(10)*(Q**4) + 2*RMvcs(1,2)*Ur(31)*(Q**5)) + 
     6   Jem(3,3)*(-2*RMvcs(3,3)*Ur(11)*qsq - 
     7   2*RMvcs(3,1)*Ur(10)*(Q**4) + 2*RMvcs(3,2)*Ur(31)*(Q**5)) + 
     8   Jem(2,3)*(-2*RMvcs(2,3)*Ur(52)*qsq + 
     9   2*RMvcs(2,1)*Ur(17)*(Q**4) + 2*RMvcs(2,2)*Ur(85)*(Q**5)) + 
     1   Jem(4,3)*(-2*RMvcs(4,3)*Ur(52)*qsq + 
     2   2*RMvcs(4,1)*Ur(17)*(Q**4) + 2*RMvcs(4,2)*Ur(85)*(Q**5))

      SigmaIPol0(6)=Jem(1,2)*(-2*RMvcs(1,3)*Ur(42)*Q - 
     1   2*RMvcs(1,1)*Ur(5)*(Q**3)) + 
     2   Jem(3,2)*(-2*RMvcs(3,3)*Ur(42)*Q - 
     3   2*RMvcs(3,1)*Ur(5)*(Q**3)) + 
     4   Jem(1,3)*(-2*RMvcs(1,3)*Ur(48)*Q - 
     5   2*RMvcs(1,1)*Ur(13)*(Q**5)) + 
     6   Jem(3,3)*(-2*RMvcs(3,3)*Ur(48)*Q - 
     7   2*RMvcs(3,1)*Ur(13)*(Q**5)) + 
     8   Jem(2,3)*(-2*RMvcs(2,3)*Ur(54)*Q + 
     9   2*RMvcs(2,1)*Ur(19)*(Q**5)) + 
     1   Jem(4,3)*(-2*RMvcs(4,3)*Ur(54)*Q + 
     2   2*RMvcs(4,1)*Ur(19)*(Q**5))

      SigmaIPol0(7)=Jem(1,2)*(-2*IMvcs(1,2)*Ur(29)*qsq + 
     1   2*IMvcs(1,3)*Ur(45)*(Q**3) + 2*IMvcs(1,1)*Ur(8)*(Q**3)) + 
     2   Jem(3,2)*(-2*IMvcs(3,2)*Ur(29)*qsq + 
     3   2*IMvcs(3,3)*Ur(45)*(Q**3) + 2*IMvcs(3,1)*Ur(8)*(Q**3)) + 
     4   Jem(1,3)*(2*IMvcs(1,1)*Ur(16)*(Q**3) - 
     5   2*IMvcs(1,2)*Ur(34)*(Q**4) + 2*IMvcs(1,3)*Ur(51)*(Q**5)) + 
     6   Jem(3,3)*(2*IMvcs(3,1)*Ur(16)*(Q**3) - 
     7   2*IMvcs(3,2)*Ur(34)*(Q**4) + 2*IMvcs(3,3)*Ur(51)*(Q**5)) + 
     8   Jem(2,3)*(2*IMvcs(2,1)*Ur(73)*(Q**3) + 
     9   2*IMvcs(2,2)*Ur(86)*(Q**4) + 2*IMvcs(2,3)*Ur(100)*(Q**5)) + 
     1   Jem(4,3)*(2*IMvcs(4,1)*Ur(73)*(Q**3) + 
     2   2*IMvcs(4,2)*Ur(86)*(Q**4) + 2*IMvcs(4,3)*Ur(100)*(Q**5))

      SigmaIPol0(8)=Jem(1,2)*(2*IMvcs(1,3)*Ur(44)*qsq - 
     1   4*IMvcs(1,2)*Ur(26)*(Q**3) + 2*IMvcs(1,1)*Ur(7)*(Q**4)) + 
     2   Jem(3,2)*(2*IMvcs(3,3)*Ur(44)*qsq - 
     3   4*IMvcs(3,2)*Ur(26)*(Q**3) + 2*IMvcs(3,1)*Ur(7)*(Q**4)) + 
     4   Jem(1,3)*(2*IMvcs(1,3)*Ur(50)*qsq + 
     5   2*IMvcs(1,1)*Ur(15)*(Q**4) - 4*IMvcs(1,2)*Ur(31)*(Q**5)) + 
     6   Jem(3,3)*(2*IMvcs(3,3)*Ur(50)*qsq + 
     7   2*IMvcs(3,1)*Ur(15)*(Q**4) - 4*IMvcs(3,2)*Ur(31)*(Q**5)) + 
     8   Jem(2,3)*(2*IMvcs(2,3)*Ur(57)*qsq - 
     9   2*IMvcs(2,1)*Ur(22)*(Q**4) - 4*IMvcs(2,2)*Ur(85)*(Q**5)) + 
     1   Jem(4,3)*(2*IMvcs(4,3)*Ur(57)*qsq - 
     2   2*IMvcs(4,1)*Ur(22)*(Q**4) - 4*IMvcs(4,2)*Ur(85)*(Q**5))

      SigmaIPolX(1)=Jem(1,2)*(2*IMvcs(2,1)*Ur(60)*(Q**3) + 
     1    2*IMvcs(2,3)*Ur(87)*(Q**3) - 2*IMvcs(2,2)*Ur(75)*(Q**4)) + 
     2    Jem(3,2)*(-2*IMvcs(4,1)*Ur(60)*(Q**3) - 
     3    2*IMvcs(4,3)*Ur(87)*(Q**3) + 2*IMvcs(4,2)*Ur(75)*(Q**4)) + 
     4    Jem(2,3)*(2*IMvcs(1,1)*Ur(18)*(Q**3) - 
     5    2*IMvcs(1,2)*Ur(36)*(Q**4) + 2*IMvcs(1,3)*Ur(53)*(Q**5)) + 
     6    Jem(4,3)*(-2*IMvcs(3,1)*Ur(18)*(Q**3) + 
     7    2*IMvcs(3,2)*Ur(36)*(Q**4) - 2*IMvcs(3,3)*Ur(53)*(Q**5)) + 
     8    Jem(1,3)*(2*IMvcs(2,1)*Ur(65)*(Q**3) - 
     9    2*IMvcs(2,2)*Ur(80)*(Q**4) + 2*IMvcs(2,3)*Ur(92)*(Q**5)) + 
     1    Jem(3,3)*(-2*IMvcs(4,1)*Ur(65)*(Q**3) + 
     2    2*IMvcs(4,2)*Ur(80)*(Q**4) - 2*IMvcs(4,3)*Ur(92)*(Q**5))

      SigmaIPolX(2)=Jem(1,2)*(2*IMvcs(2,1)*Ur(61)*(Q**3) + 
     1    2*IMvcs(2,3)*Ur(88)*(Q**3) - 2*IMvcs(2,2)*Ur(76)*(Q**4)) + 
     2    Jem(3,2)*(-2*IMvcs(4,1)*Ur(61)*(Q**3) - 
     3    2*IMvcs(4,3)*Ur(88)*(Q**3) + 2*IMvcs(4,2)*Ur(76)*(Q**4)) + 
     4    Jem(2,3)*(2*IMvcs(1,1)*Ur(20)*(Q**3) - 
     5    2*IMvcs(1,2)*Ur(37)*(Q**4) + 2*IMvcs(1,3)*Ur(55)*(Q**5)) + 
     6    Jem(4,3)*(-2*IMvcs(3,1)*Ur(20)*(Q**3) + 
     7    2*IMvcs(3,2)*Ur(37)*(Q**4) - 2*IMvcs(3,3)*Ur(55)*(Q**5)) + 
     8    Jem(1,3)*(2*IMvcs(2,1)*Ur(66)*(Q**3) - 
     9    2*IMvcs(2,2)*Ur(81)*(Q**4) + 2*IMvcs(2,3)*Ur(93)*(Q**5)) + 
     1    Jem(3,3)*(-2*IMvcs(4,1)*Ur(66)*(Q**3) + 
     2    2*IMvcs(4,2)*Ur(81)*(Q**4) - 2*IMvcs(4,3)*Ur(93)*(Q**5))

      SigmaIPolX(3)=Jem(1,2)*(2*IMvcs(2,3)*Ur(3)*qsq - 
     1    2*IMvcs(2,2)*Ur(74)*(Q**3) - 2*IMvcs(2,1)*Ur(2)*(Q**4)) + 
     2    Jem(3,2)*(-2*IMvcs(4,3)*Ur(3)*qsq + 
     3    2*IMvcs(4,2)*Ur(74)*(Q**3) + 2*IMvcs(4,1)*Ur(2)*(Q**4)) + 
     4    Jem(2,3)*(2*IMvcs(1,3)*Ur(52)*qsq + 
     5    2*IMvcs(1,1)*Ur(17)*(Q**4) - 2*IMvcs(1,2)*Ur(35)*(Q**5)) + 
     6    Jem(4,3)*(-2*IMvcs(3,3)*Ur(52)*qsq - 
     7    2*IMvcs(3,1)*Ur(17)*(Q**4) + 2*IMvcs(3,2)*Ur(35)*(Q**5)) + 
     8    Jem(1,3)*(2*IMvcs(2,3)*Ur(11)*qsq - 
     9    2*IMvcs(2,1)*Ur(10)*(Q**4) - 2*IMvcs(2,2)*Ur(79)*(Q**5)) + 
     1    Jem(3,3)*(-2*IMvcs(4,3)*Ur(11)*qsq + 
     2    2*IMvcs(4,1)*Ur(10)*(Q**4) + 2*IMvcs(4,2)*Ur(79)*(Q**5))

      SigmaIPolX(4)=Jem(1,2)*(2*IMvcs(2,3)*Ur(42)*Q - 
     1    2*IMvcs(2,1)*Ur(5)*(Q**3)) + 
     2    Jem(3,2)*(-2*IMvcs(4,3)*Ur(42)*Q + 
     3    2*IMvcs(4,1)*Ur(5)*(Q**3)) + 
     4    Jem(1,3)*(2*IMvcs(2,3)*Ur(48)*Q - 
     5    2*IMvcs(2,1)*Ur(13)*(Q**5)) + 
     6    Jem(3,3)*(-2*IMvcs(4,3)*Ur(48)*Q + 
     7    2*IMvcs(4,1)*Ur(13)*(Q**5)) + 
     8    Jem(2,3)*(2*IMvcs(1,3)*Ur(54)*Q + 
     9    2*IMvcs(1,1)*Ur(19)*(Q**5)) + 
     1    Jem(4,3)*(-2*IMvcs(3,3)*Ur(54)*Q - 
     2    2*IMvcs(3,1)*Ur(19)*(Q**5))

      SigmaIPolX(5)=Jem(2,3)*(-2*RMvcs(1,1)*Ur(11)*qsq + 
     1    2*RMvcs(1,2)*Ur(38)*(Q**3) - 2*RMvcs(1,3)*Ur(56)*(Q**4)) + 
     2    Jem(4,3)*(2*RMvcs(3,1)*Ur(21)*qsq - 
     3    2*RMvcs(3,2)*Ur(38)*(Q**3) + 2*RMvcs(3,3)*Ur(56)*(Q**4)) + 
     4    Jem(1,2)*(-2*RMvcs(2,1)*Ur(62)*qsq + 
     5    2*RMvcs(2,2)*Ur(77)*(Q**3) - 2*RMvcs(2,3)*Ur(89)*(Q**4)) + 
     6    Jem(3,2)*(2*RMvcs(4,1)*Ur(62)*qsq - 
     7    2*RMvcs(4,2)*Ur(77)*(Q**3) + 2*RMvcs(4,3)*Ur(89)*(Q**4)) + 
     8    Jem(1,3)*(-2*RMvcs(2,1)*Ur(67)*qsq + 
     9    2*RMvcs(2,2)*Ur(82)*(Q**3) - 2*RMvcs(2,3)*Ur(94)*(Q**4)) + 
     1    Jem(3,3)*(2*RMvcs(4,1)*Ur(67)*qsq - 
     2    2*RMvcs(4,2)*Ur(82)*(Q**3) + 2*RMvcs(4,3)*Ur(94)*(Q**4))

      SigmaIPolX(6)=Jem(2,3)*(-2*RMvcs(1,1)*Ur(23)*qsq - 
     1    2*RMvcs(1,2)*Ur(38)*(Q**3) - 2*RMvcs(1,3)*Ur(58)*(Q**4)) + 
     2    Jem(4,3)*(2*RMvcs(3,1)*Ur(23)*qsq + 
     3    2*RMvcs(3,2)*Ur(38)*(Q**3) + 2*RMvcs(3,3)*Ur(58)*(Q**4)) + 
     4    Jem(1,2)*(-2*RMvcs(2,1)*Ur(63)*qsq - 
     5    2*RMvcs(2,2)*Ur(77)*(Q**3) - 2*RMvcs(2,3)*Ur(90)*(Q**4)) + 
     6    Jem(3,2)*(2*RMvcs(4,1)*Ur(63)*qsq + 
     7    2*RMvcs(4,2)*Ur(77)*(Q**3) + 2*RMvcs(4,3)*Ur(90)*(Q**4)) + 
     8    Jem(1,3)*(-2*RMvcs(2,1)*Ur(68)*qsq - 
     9    2*RMvcs(2,2)*Ur(82)*(Q**3) - 2*RMvcs(2,3)*Ur(95)*(Q**4)) + 
     1    Jem(3,3)*(2*RMvcs(4,1)*Ur(68)*qsq + 
     2    2*RMvcs(4,2)*Ur(82)*(Q**3) + 2*RMvcs(4,3)*Ur(95)*(Q**4))

      SigmaIPolX(7)=Jem(1,2)*(2*RMvcs(2,2)*Ur(78)*qsq - 
     1    2*RMvcs(2,1)*Ur(64)*(Q**3) - 2*RMvcs(2,3)*Ur(91)*(Q**3)) + 
     2    Jem(3,2)*(-2*RMvcs(4,2)*Ur(78)*qsq + 
     3    2*RMvcs(4,1)*Ur(64)*(Q**3) + 2*RMvcs(4,3)*Ur(91)*(Q**3)) + 
     4    Jem(2,3)*(-2*RMvcs(1,1)*Ur(24)*(Q**3) + 
     5    2*RMvcs(1,2)*Ur(39)*(Q**4) - 2*RMvcs(1,3)*Ur(59)*(Q**5)) + 
     6    Jem(4,3)*(2*RMvcs(3,1)*Ur(24)*(Q**3) - 
     7    2*RMvcs(3,2)*Ur(39)*(Q**4) + 2*RMvcs(3,3)*Ur(59)*(Q**5)) + 
     8    Jem(1,3)*(-2*RMvcs(2,1)*Ur(69)*(Q**3) + 
     9    2*RMvcs(2,2)*Ur(83)*(Q**4) - 2*RMvcs(2,3)*Ur(96)*(Q**5)) + 
     1    Jem(3,3)*(2*RMvcs(4,1)*Ur(69)*(Q**3) - 
     2    2*RMvcs(4,2)*Ur(83)*(Q**4) + 2*RMvcs(4,3)*Ur(96)*(Q**5))

      SigmaIPolX(8)=Jem(1,2)*(-2*RMvcs(2,3)*Ur(44)*qsq + 
     1    4*RMvcs(2,2)*Ur(74)*(Q**3) + 2*RMvcs(2,1)*Ur(7)*(Q**4)) + 
     2    Jem(3,2)*(2*RMvcs(4,3)*Ur(44)*qsq - 
     3    4*RMvcs(4,2)*Ur(74)*(Q**3) - 2*RMvcs(4,1)*Ur(7)*(Q**4)) + 
     4    Jem(2,3)*(-2*RMvcs(1,3)*Ur(57)*qsq - 
     5    2*RMvcs(1,1)*Ur(22)*(Q**4) + 4*RMvcs(1,2)*Ur(35)*(Q**5)) + 
     6    Jem(4,3)*(2*RMvcs(3,3)*Ur(57)*qsq + 
     7    2*RMvcs(3,1)*Ur(22)*(Q**4) - 4*RMvcs(3,2)*Ur(35)*(Q**5)) + 
     8    Jem(1,3)*(-2*RMvcs(2,3)*Ur(50)*qsq + 
     9    2*RMvcs(2,1)*Ur(15)*(Q**4) + 4*RMvcs(2,2)*Ur(79)*(Q**5)) + 
     1    Jem(3,3)*(2*RMvcs(4,3)*Ur(50)*qsq - 
     2    2*RMvcs(4,1)*Ur(15)*(Q**4) - 4*RMvcs(4,2)*Ur(79)*(Q**5))

      SigmaIPolY(1)=Jem(3,2)*(2*IMvcs(1,1)*Ur(1)*qsq - 
     1    2*IMvcs(1,2)*Ur(25)*(Q**3) + 2*IMvcs(1,3)*Ur(40)*(Q**4)) + 
     2    Jem(1,2)*(-2*IMvcs(3,1)*Ur(1)*qsq + 
     3    2*IMvcs(3,2)*Ur(25)*(Q**3) - 2*IMvcs(3,3)*Ur(40)*(Q**4)) + 
     4    Jem(3,3)*(2*IMvcs(1,1)*Ur(9)*qsq - 
     5    2*IMvcs(1,2)*Ur(30)*(Q**3) + 2*IMvcs(1,3)*Ur(46)*(Q**4)) + 
     6    Jem(1,3)*(-2*IMvcs(3,1)*Ur(9)*qsq + 
     7    2*IMvcs(3,2)*Ur(30)*(Q**3) - 2*IMvcs(3,3)*Ur(46)*(Q**4)) + 
     8    Jem(4,3)*(2*IMvcs(2,1)*Ur(70)*qsq - 
     9    2*IMvcs(2,2)*Ur(84)*(Q**3) + 2*IMvcs(2,3)*Ur(97)*(Q**4)) + 
     1    Jem(2,3)*(-2*IMvcs(4,1)*Ur(70)*qsq + 
     2    2*IMvcs(4,2)*Ur(84)*(Q**3) - 2*IMvcs(4,3)*Ur(97)*(Q**4))

      SigmaIPolY(2)=Jem(3,2)*(2*IMvcs(1,1)*Ur(3)*qsq + 
     1    2*IMvcs(1,2)*Ur(25)*(Q**3) + 2*IMvcs(1,3)*Ur(2)*(Q**4)) + 
     2    Jem(1,2)*(-2*IMvcs(3,1)*Ur(3)*qsq - 
     3    2*IMvcs(3,2)*Ur(25)*(Q**3) - 2*IMvcs(3,3)*Ur(2)*(Q**4)) + 
     4    Jem(4,3)*(2*IMvcs(2,1)*Ur(52)*qsq + 
     5    2*IMvcs(2,2)*Ur(84)*(Q**3) - 2*IMvcs(2,3)*Ur(17)*(Q**4)) + 
     6    Jem(2,3)*(-2*IMvcs(4,1)*Ur(52)*qsq - 
     7    2*IMvcs(4,2)*Ur(84)*(Q**3) + 2*IMvcs(4,3)*Ur(17)*(Q**4)) + 
     8    Jem(3,3)*(2*IMvcs(1,1)*Ur(11)*qsq + 
     9    2*IMvcs(1,2)*Ur(30)*(Q**3) + 2*IMvcs(1,3)*Ur(10)*(Q**4)) + 
     1    Jem(1,3)*(-2*IMvcs(3,1)*Ur(11)*qsq - 
     2    2*IMvcs(3,2)*Ur(30)*(Q**3) - 2*IMvcs(3,3)*Ur(10)*(Q**4))

      SigmaIPolY(3)=Jem(3,2)*(2*IMvcs(1,1)*Ur(4)*(Q**3) + 
     1    2*IMvcs(1,3)*Ur(41)*(Q**3) - 2*IMvcs(1,2)*Ur(27)*(Q**4)) + 
     2    Jem(1,2)*(-2*IMvcs(3,1)*Ur(4)*(Q**3) - 
     3    2*IMvcs(3,3)*Ur(41)*(Q**3) + 2*IMvcs(3,2)*Ur(27)*(Q**4)) + 
     4    Jem(3,3)*(2*IMvcs(1,1)*Ur(12)*(Q**3) - 
     5    2*IMvcs(1,2)*Ur(32)*(Q**4) + 2*IMvcs(1,3)*Ur(47)*(Q**5)) + 
     6    Jem(1,3)*(-2*IMvcs(3,1)*Ur(12)*(Q**3) + 
     7    2*IMvcs(3,2)*Ur(32)*(Q**4) - 2*IMvcs(3,3)*Ur(47)*(Q**5)) + 
     8    Jem(4,3)*(2*IMvcs(2,1)*Ur(71)*(Q**3) - 
     9    2*IMvcs(2,2)*Ur(86)*(Q**4) + 2*IMvcs(2,3)*Ur(98)*(Q**5)) + 
     1    Jem(2,3)*(-2*IMvcs(4,1)*Ur(71)*(Q**3) + 
     2    2*IMvcs(4,2)*Ur(86)*(Q**4) - 2*IMvcs(4,3)*Ur(98)*(Q**5))

      SigmaIPolY(4)=Jem(3,2)*(2*IMvcs(1,3)*Ur(43)*(Q**3) + 
     1    2*IMvcs(1,1)*Ur(6)*(Q**3) - 2*IMvcs(1,2)*Ur(28)*(Q**4)) + 
     2    Jem(1,2)*(-2*IMvcs(3,3)*Ur(43)*(Q**3) - 
     3    2*IMvcs(3,1)*Ur(6)*(Q**3) + 2*IMvcs(3,2)*Ur(28)*(Q**4)) + 
     4    Jem(3,3)*(2*IMvcs(1,1)*Ur(14)*(Q**3) - 
     5    2*IMvcs(1,2)*Ur(33)*(Q**4) + 2*IMvcs(1,3)*Ur(49)*(Q**5)) + 
     6    Jem(1,3)*(-2*IMvcs(3,1)*Ur(14)*(Q**4) + 
     7    2*IMvcs(3,2)*Ur(33)*(Q**4) - 2*IMvcs(3,3)*Ur(49)*(Q**5)) + 
     8    Jem(4,3)*(2*IMvcs(2,1)*Ur(72)*(Q**3) + 
     9    2*IMvcs(2,2)*Ur(86)*(Q**4) + 2*IMvcs(2,3)*Ur(99)*(Q**5)) + 
     1    Jem(2,3)*(-2*IMvcs(4,1)*Ur(72)*(Q**3) - 
     2    2*IMvcs(4,2)*Ur(86)*(Q**4) - 2*IMvcs(4,3)*Ur(99)*(Q**5))

      SigmaIPolY(5)=Jem(3,2)*(2*IMvcs(1,3)*Ur(3)*qsq - 
     1    2*IMvcs(1,2)*Ur(26)*(Q**3) + 2*IMvcs(1,1)*Ur(2)*(Q**4)) + 
     2    Jem(1,2)*(-2*IMvcs(3,3)*Ur(3)*qsq + 
     3    2*IMvcs(3,2)*Ur(26)*(Q**3) - 2*IMvcs(3,1)*Ur(2)*(Q**4)) + 
     4    Jem(3,3)*(2*IMvcs(1,3)*Ur(11)*qsq + 
     5    2*IMvcs(1,1)*Ur(10)*(Q**4) - 2*IMvcs(1,2)*Ur(31)*(Q**5)) + 
     6    Jem(1,3)*(-2*IMvcs(3,3)*Ur(11)*qsq - 
     7    2*IMvcs(3,1)*Ur(10)*(Q**4) + 2*IMvcs(3,2)*Ur(31)*(Q**5)) + 
     8    Jem(4,3)*(2*IMvcs(2,3)*Ur(52)*qsq - 
     9    2*IMvcs(2,1)*Ur(17)*(Q**4) - 2*IMvcs(2,2)*Ur(85)*(Q**5)) + 
     1    Jem(2,3)*(-2*IMvcs(4,3)*Ur(52)*qsq + 
     2    2*IMvcs(4,1)*Ur(177)*(Q**4) + 2*IMvcs(4,2)*Ur(85)*(Q**5))

      SigmaIPolY(6)=Jem(3,2)*(2*IMvcs(1,3)*Ur(42)*Q + 
     1    2*IMvcs(1,1)*Ur(5)*(Q**3)) + 
     2    Jem(1,2)*(-2*IMvcs(3,3)*Ur(42)*Q - 
     3    2*IMvcs(3,1)*Ur(5)*(Q**3)) + 
     4    Jem(3,3)*(2*IMvcs(1,3)*Ur(48)*Q + 
     5    2*IMvcs(1,1)*Ur(13)*(Q**5)) + 
     6    Jem(1,3)*(-2*IMvcs(3,3)*Ur(48)*Q - 
     7    2*IMvcs(3,1)*Ur(13)*(Q**5)) + 
     8    Jem(4,3)*(2*IMvcs(2,3)*Ur(54)*Q - 
     9    2*IMvcs(2,1)*Ur(19)*(Q**5)) + 
     1    Jem(2,3)*(-2*IMvcs(4,3)*Ur(54)*Q + 
     2    2*IMvcs(4,1)*Ur(19)*(Q**5))

      SigmaIPolY(7)=Jem(3,2)*(-2*RMvcs(1,2)*Ur(29)*qsq + 
     1    2*RMvcs(1,3)*Ur(45)*(Q**3) + 2*RMvcs(1,1)*Ur(8)*(Q**3)) + 
     2    Jem(1,2)*(2*RMvcs(3,2)*Ur(29)*qsq - 
     3    2*RMvcs(3,3)*Ur(45)*(Q**3) - 2*RMvcs(3,1)*Ur(8)*(Q**3)) + 
     4    Jem(3,3)*(2*RMvcs(1,1)*Ur(16)*(Q**3) - 
     5    2*RMvcs(1,2)*Ur(34)*(qsq**2) + 2*RMvcs(1,3)*Ur(51)*(Q**5)) + 
     6    Jem(1,3)*(-2*RMvcs(3,1)*Ur(16)*(Q**3) + 
     7    2*RMvcs(3,2)*Ur(34)*(qsq**2) - 2*RMvcs(3,3)*Ur(51)*(Q**5)) + 
     8    Jem(4,3)*(2*RMvcs(2,1)*Ur(73)*(Q**3) + 
     9    2*RMvcs(2,2)*Ur(86)*(qsq**2) + 2*RMvcs(2,3)*Ur(100)*(Q**5)) + 
     1    Jem(2,3)*(-2*RMvcs(4,1)*Ur(73)*(Q**3) - 
     2    2*RMvcs(4,2)*Ur(86)*(qsq**2) - 2*RMvcs(4,3)*Ur(100)*(Q**5))

      SigmaIPolY(8)=Jem(3,2)*(2*RMvcs(1,3)*Ur(44)*qsq - 
     1    4*RMvcs(1,2)*Ur(26)*(Q**3) + 2*RMvcs(1,1)*Ur(7)*(qsq**2)) + 
     2    Jem(1,2)*(-2*RMvcs(3,3)*Ur(44)*qsq + 
     3    4*RMvcs(3,2)*Ur(26)*(Q**3) - 2*RMvcs(3,1)*Ur(7)*(qsq**2)) + 
     4    Jem(3,3)*(2*RMvcs(1,3)*Ur(50)*qsq + 2*RMvcs(1,1)*Ur(15)*(qsq**2) - 
     5    4*RMvcs(1,2)*Ur(31)*(Q**5)) + 
     6    Jem(1,3)*(-2*RMvcs(3,3)*Ur(50)*qsq - 
     7    2*RMvcs(3,1)*Ur(15)*(qsq**2) + 4*RMvcs(3,2)*Ur(41)*(Q**5)) + 
     8    Jem(4,3)*(2*RMvcs(2,3)*Ur(57)*qsq - 
     9    2*RMvcs(2,1)*Ur(22)*(qsq**2) - 4*RMvcs(2,2)*Ur(85)*(Q**5)) + 
     1    Jem(2,3)*(-2*RMvcs(4,3)*Ur(57)*qsq + 
     2    2*RMvcs(4,1)*Ur(22)*(qsq**2) + 4*RMvcs(4,2)*Ur(85)*(Q**5))

      SigmaIPolZ(1)=Jem(3,2)*(2*IMvcs(2,1)*Ur(60)*(Q**3) + 
     1    2*IMvcs(2,3)*Ur(87)*(Q**3) - 2*IMvcs(2,2)*Ur(75)*(qsq**2)) + 
     2    Jem(1,2)*(2*IMvcs(4,1)*Ur(60)*(Q**3) + 
     3    2*IMvcs(4,3)*Ur(87)*(Q**3) - 2*IMvcs(4,2)*Ur(75)*(qsq**2)) + 
     4    Jem(4,3)*(2*IMvcs(1,1)*Ur(18)*(Q**3) - 
     5    2*IMvcs(1,2)*Ur(36)*(qsq**2) + 2*IMvcs(1,3)*Ur(53)*(Q**5)) + 
     6    Jem(2,3)*(2*IMvcs(3,1)*Ur(18)*(Q**3) - 
     7    2*IMvcs(3,2)*Ur(36)*(qsq**2) + 2*IMvcs(3,3)*Ur(53)*(Q**5)) + 
     8    Jem(3,3)*(2*IMvcs(2,1)*Ur(65)*(Q**3) - 
     9    2*IMvcs(2,2)*Ur(80)*(qsq**2) + 2*IMvcs(2,3)*Ur((2)*(Q**5)) + 
     1    Jem(1,3)*(2*IMvcs(4,1)*Ur(65)*(Q**3) - 
     2    2*IMvcs(4,2)*Ur(80)*(qsq**2) + 2*IMvcs(4,3)*Ur(92)*(Q**5))

      SigmaIPolZ(2)=Jem(3,2)*(2*IMvcs(2,1)*Ur(61)*(Q**3) + 
     1    2*IMvcs(2,3)*Ur(88)*(Q**3) - 2*IMvcs(2,2)*Ur(76)*(qsq**2)) + 
     2    Jem(1,2)*(2*IMvcs(4,1)*Ur(61)*(Q**3) + 
     3    2*IMvcs(4,3)*Ur(88)*(Q**3) - 2*IMvcs(4,2)*Ur(76)*(qsq**2)) + 
     4    Jem(4,3)*(2*IMvcs(1,1)*Ur(20)*(Q**3) - 
     5    2*IMvcs(1,2)*Ur(37)*(qsq**2) + 2*IMvcs(1,3)*Ur(55)*(Q**5)) + 
     6    Jem(2,3)*(2*IMvcs(3,1)*Ur(20)*(Q**3) - 
     7    2*IMvcs(3,2)*Ur(37)*(qsq**2) + 2*IMvcs(3,3)*Ur(55)*(Q**5)) + 
     8    Jem(3,3)*(2*IMvcs(2,1)*Ur(66)*(Q**3) - 
     9    2*IMvcs(2,2)*Ur(81)*(qsq**2) + 2*IMvcs(2,3)*Ur(93)*(Q**5)) + 
     1    Jem(1,3)*(2*IMvcs(4,1)*Ur(66)*(Q**3) - 
     2    2*IMvcs(4,2)*Ur(81)*(qsq**2) + 2*IMvcs(4,3)*Ur(93)*(Q**5))

      SigmaIPolZ(3)=Jem(3,2)*(2*IMvcs(2,3)*Ur(3)*qsq - 
     1    2*IMvcs(2,2)*Ur(74)*(Q**3) - 2*IMvcs(2,1)*Ur(2)*(qsq**2)) + 
     2    Jem(1,2)*(2*IMvcs(4,3)*Ur(3)*qsq - 
     3    2*IMvcs(4,2)*Ur(74)*(Q**3) - 2*IMvcs(4,1)*Ur(2)*(qsq**2)) + 
     4    Jem(4,3)*(2*IMvcs(1,3)*Ur(52)*qsq + 
     5    2*IMvcs(1,1)*Ur(17)*(qsq**2) - 2*IMvcs(1,2)*Ur(35)*(Q**5)) + 
     6    Jem(2,3)*(2*IMvcs(3,3)*Ur(52)*qsq + 
     7    2*IMvcs(3,1)*Ur(17)*(qsq**2) - 2*IMvcs(3,2)*Ur(35)*(Q**5)) + 
     8    Jem(3,3)*(2*IMvcs(2,3)*Ur(11)*qsq - 
     9    2*IMvcs(2,1)*Ur(10)*(qsq**2) - 2*IMvcs(2,2)*Ur(79)*(Q**5)) + 
     1    Jem(1,3)*(2*IMvcs(4,3)*Ur(11)*qsq - 
     2    2*IMvcs(4,1)*Ur(10)*(qsq**2) - 2*IMvcs(4,2)*Ur(79)*(Q**5))

      SigmaIPolZ(4)=Jem(3,2)*(2*IMvcs(2,3)*Ur(42)*Q - 
     1    2*IMvcs(2,1)*Ur(5)*(Q**3)) + 
     2    Jem(1,2)*(2*IMvcs(4,3)*Ur(42)*Q - 
     3    2*IMvcs(4,1)*Ur(5)*(Q**3)) + 
     4    Jem(3,3)*(2*IMvcs(2,3)*Ur(48)*Q - 
     5    2*IMvcs(2,1)*Ur(13)*(Q**5)) + 
     6    Jem(1,3)*(2*IMvcs(4,3)*Ur(48)*Q - 
     7    2*IMvcs(4,1)*Ur(13)*(Q**5)) + 
     8    Jem(4,3)*(2*IMvcs(1,3)*Ur(54)*Q + 
     9    2*IMvcs(1,1)*Ur(19)*(Q**5)) + 
     1    Jem(2,3)*(2*IMvcs(3,3)*Ur(54)*Q + 
     2    2*IMvcs(3,1)*Ur(19)*(Q**5))

      SigmaIPolZ(5)=Jem(4,3)*(-2*RMvcs(1,1)*Ur(21)*qsq + 
     1    2*RMvcs(1,2)*Ur(38)*(Q**3) - 2*RMvcs(1,3)*Ur(56)*(qsq**2)) + 
     2    Jem(2,3)*(-2*RMvcs(3,1)*Ur(21)*qsq + 
     3    2*RMvcs(3,2)*Ur(38)*(Q**3) - 2*RMvcs(3,3)*Ur(56)*(qsq**2)) + 
     4    Jem(3,2)*(-2*RMvcs(2,1)*Ur(62)*qsq + 
     5    2*RMvcs(2,2)*Ur(77)*(Q**3) - 2*RMvcs(2,3)*Ur(89)*(qsq**2)) + 
     6    Jem(1,2)*(-2*RMvcs(4,1)*Ur(62)*qsq + 
     7    2*RMvcs(4,2)*Ur(77)*(Q**3) - 2*RMvcs(4,3)*Ur(89)*(qsq**2)) + 
     8    Jem(3,3)*(-2*RMvcs(2,1)*Ur(67)*qsq + 
     9    2*RMvcs(2,2)*Ur(82)*(Q**3) - 2*RMvcs(2,3)*Ur(94)*(qsq**2)) + 
     1    Jem(1,3)*(-2*RMvcs(4,1)*Ur(67)*qsq + 
     2    2*RMvcs(4,2)*Ur(82)*(Q**3) - 2*RMvcs(4,3)*Ur(94)*(qsq**2))

      SigmaIPolZ(6)=Jem(4,3)*(-2*RMvcs(1,1)*Ur(23)*qsq - 
     1    2*RMvcs(1,2)*Ur(38)*(Q**3) - 2*RMvcs(1,3)*Ur(58)*(qsq**2)) + 
     2    Jem(2,3)*(-2*RMvcs(3,1)*Ur(23)*qsq - 
     3    2*RMvcs(3,2)*Ur(38)*(Q**3) - 2*RMvcs(3,3)*Ur(58)*(qsq**2)) + 
     4    Jem(3,2)*(-2*RMvcs(2,1)*Ur(63)*qsq - 
     5    2*RMvcs(2,2)*Ur(77)*(Q**3) - 2*RMvcs(2,3)*Ur(90)*(qsq**2)) + 
     6    Jem(1,2)*(-2*RMvcs(4,1)*Ur(63)*qsq - 
     7    2*RMvcs(4,2)*Ur(77)*(Q**3) - 2*RMvcs(4,3)*Ur(90)*(qsq**2)) + 
     8    Jem(3,3)*(-2*RMvcs(2,1)*Ur(68)*qsq - 
     9    2*RMvcs(2,2)*Ur(82)*(Q**3) - 2*RMvcs(2,3)*Ur(95)*(qsq**2)) + 
     1    Jem(1,3)*(-2*RMvcs(4,1)*Ur(68)*qsq - 
     2    2*RMvcs(4,2)*Ur(82)*(Q**3) - 2*RMvcs(4,3)*Ur(95)*(qsq**2))

      SigmaIPolZ(7)=Jem(3,2)*(2*RMvcs(2,2)*Ur(78)*qsq - 
     1    2*RMvcs(2,1)*Ur(64)*(Q**3) - 2*RMvcs(2,3)*Ur(91)*(Q**3)) + 
     2    Jem(1,2)*(2*RMvcs(4,2)*Ur(78)*qsq - 
     3    2*RMvcs(4,1)*Ur(64)*(Q**3) - 2*RMvcs(4,3)*Ur(91)*(Q**3)) + 
     4    Jem(4,3)*(-2*RMvcs(1,1)*Ur(24)*(Q**3) + 
     5    2*RMvcs(1,2)*Ur(39)*(qsq**2) - 2*RMvcs(1,3)*Ur(59)*(Q**5)) + 
     6    Jem(2,3)*(-2*RMvcs(3,1)*Ur(24)*(Q**3) + 
     7    2*RMvcs(3,2)*Ur(39)*(qsq**2) - 2*RMvcs(3,3)*Ur(59)*(Q**5)) + 
     8    Jem(3,3)*(-2*RMvcs(2,1)*Ur(69)*(Q**3) + 
     9    2*RMvcs(2,2)*Ur(83)*(qsq**2) - 2*RMvcs(2,3)*Ur(96)*(Q**5)) + 
     1    Jem(1,3)*(-2*RMvcs(4,1)*Ur(69)*(Q**3) + 
     2    2*RMvcs(4,2)*Ur(83)*(qsq**2) - 2*RMvcs(4,3)*Ur(96)*(Q**5))

      SigmaIPolZ(8)=Jem(3,2)*(-2*RMvcs(2,3)*Ur(44)*qsq + 
     1    4*RMvcs(2,2)*Ur(74)*(Q**3) + 2*RMvcs(2,1)*Ur(7)*(qsq**2)) + 
     2    Jem(1,2)*(-2*RMvcs(4,3)*Ur(44)*qsq + 
     3    4*RMvcs(4,2)*Ur(74)*(Q**3) + 2*RMvcs(4,1)*Ur(7)*(qsq**2)) + 
     4    Jem(4,3)*(-2*RMvcs(1,3)*Ur(57)*qsq - 
     5    2*RMvcs(1,1)*Ur(22)*(qsq**2) + 4*RMvcs(1,2)*Ur(35)*(Q**5)) + 
     6    Jem(2,3)*(-2*RMvcs(3,3)*Ur(57)*qsq - 
     7    2*RMvcs(3,1)*Ur(22)*(qsq**2) + 4*RMvcs(3,2)*Ur(35)*(Q**5)) + 
     8    Jem(3,3)*(-2*RMvcs(2,3)*Ur(50)*qsq + 
     9    2*RMvcs(2,1)*Ur(15)*(qsq**2) + 4*RMvcs(2,2)*Ur(79)*(Q**5)) + 
     1    Jem(1,3)*(-2*RMvcs(4,3)*Ur(50)*qsq + 
     2    2*RMvcs(4,1)*Ur(15)*(qsq**2) + 4*RMvcs(4,2)*Ur(79)*(Q**5))
    
      if (debug(2)) then

         do j=0,90,10
	    write(6,*)j,(Ur(i+j),i=1,10)
         enddo

         write(6,*)' SigmaVCSPol0(0) ',SigmaVCSPol0(0)
         write(6,*)' SigmaVCSPolZ(0) ',SigmaVCSPolZ(0)
         write(6,*)' SigmaVCSPol0(1) ',SigmaVCSPol0(1)
         write(6,*)' SigmaVCSPolX(0) ',SigmaVCSPolX(0)
         write(6,*)' SigmaVCSPolX(1) ',SigmaVCSPolX(1)
         write(6,*)' SigmaVCSPolZ(1) ',SigmaVCSPolZ(1)
         write(6,*)' SigmaVCSPol0(2) ',SigmaVCSPol0(2)
         write(6,*)' SigmaVCSPol0(3) ',SigmaVCSPol0(3)
         write(6,*)' SigmaVCSPolY(0) ',SigmaVCSPolY(0)
         write(6,*)' SigmaVCSPolY(1) ',SigmaVCSPolY(1)
         write(6,*)' SigmaVCSPolY(2) ',SigmaVCSPolY(2)
         write(6,*)' SigmaVCSPolY(3) ',SigmaVCSPolY(3)
         write(6,*)' SigmaVCSPolY(4) ',SigmaVCSPolY(4)
         write(6,*)' SigmaVCSPol0(4) ',SigmaVCSPol0(4)
         write(6,*)' SigmaVCSPolX(2) ',SigmaVCSPolX(2)
         write(6,*)' SigmaVCSPolZ(2) ',SigmaVCSPolZ(2)
         write(6,*)' SigmaVCSPolX(3) ',SigmaVCSPolX(3)
         write(6,*)' SigmaVCSPolZ(3) ',SigmaVCSPolZ(3)
          
         write(6,*)' SigmaIPol0 ',(SigmaIPol0(i),i=1,8) 
         write(6,*)' SigmaIPolX ',(SigmaIPolX(i),i=1,8) 
         write(6,*)' SigmaIPolY ',(SigmaIPolY(i),i=1,8) 
         write(6,*)' SigmaIPolZ ',(SigmaIPolX(i),i=1,8)  

      endif
 
c----------------------------- DdirectDcrossed(phi) ----------------------------*
c Computes the denominator of the Bethe Heitler cross section.                  |
c Only depends on kinematics.                                                   |
c-------------------------------------------------------------------------------*

      DDDC=((qsq + tt)**2)/4. 
     1    - ( (qsq*(qsq + tt*(-1 + 2*xB))*CosH(Omega))/
     2        (2.*sqrt((qsq**2) + 4*(m_p**2)*qsq*(xB**2)))
     3        - Q*qpPerp*Cos(phi)*SinH(Omega) )**2

c---- SqrAmplBH(Q2Input,xBInput,tInput,phi,BeamHeli,TargetPolar,BeamCharge) ----*
c Computes the expansion of the Bethe Heitler cross section (divided by phase   |
c space) in terms of the "expansion cross sections" computed in the function    | 
c MakeExactCrossSections (or MakeLeadingCrossSections).                         |
c-------------------------------------------------------------------------------*
c need to add +1 to all indices, since fortran starts at 1, but c++ starts at 0.


      if (TargetPolar.eq.0) then
        SigmaBHp=-(SigmaBHPol0(1) - 
     1    SigmaBHPol0(4)*Cos(2*phi)*(-1 + CosH(2*Omega)) + 
     2    SigmaBHPol0(1)*CosH(2*Omega) 
     3    + SigmaBHPol0(3)*Cos(phi)*SinH(2*Omega) )/(4.*DDDC*(tt**2))

        SigmaBHm=SigmaBHp
      endif

      if (TargetPolar.eq.1) then
        BeamHeli=+1
        SigmaBHp=-(SigmaBHPolX(1)*BeamHeli*CosH(Omega) + 
     1    SigmaBHPolX(2)*BeamHeli*Cos(phi)*SinH(Omega))/(4.*DDDC*(tt**2))

        BeamHeli=-1
        SigmaBHm=-(SigmaBHPolX(1)*BeamHeli*CosH(Omega) + 
     1    SigmaBHPolX(2)*BeamHeli*Cos(phi)*SinH(Omega))/(4.*DDDC*(tt**2))
      endif
           
      if (TargetPolar.eq.2) then
        BeamHeli=+1
        SigmaBHp=-(BeamHeli*SigmaBHPolY*Sin(phi)*SinH(Omega))/
     1    (4.*DDDC*(tt**2))

        BeamHeli=-1
        SigmaBHm=-(BeamHeli*SigmaBHPolY*Sin(phi)*SinH(Omega))/
     1    (4.*DDDC*(tt**2))
      endif
     
      if (TargetPolar.eq.3) then
        BeamHeli=+1
        SigmaBHp=-(SigmaBHPolZ(1)*BeamHeli*CosH(Omega) + 
     1    SigmaBHPolZ(2)*BeamHeli*Cos(phi)*SinH(Omega))/(4.*DDDC*(tt**2))

        BeamHeli=-1
        SigmaBHm=-(SigmaBHPolZ(1)*BeamHeli*CosH(Omega) + 
     1    SigmaBHPolZ(2)*BeamHeli*Cos(phi)*SinH(Omega))/(4.*DDDC*(tt**2))
      endif
     
c---- SqrAmplVCS(Q2Input,xBInput,tInput,phi,BeamHeli,TargetPolar,BeamCharge) ---*
c Computes the expansion of the Virtual Compton Scattering cross section        |
c (divided by phase space) in terms of the "expansion cross sections" computed  |
c in the function MakeExactCrossSections (or MakeLeadingCrossSections).         |
c-------------------------------------------------------------------------------*

      if (TargetPolar.eq.0) then
        BeamHeli=+1
        SigmaVCSp=(SigmaVCSPol0(1) 
     1    - SigmaVCSPol0(4)*Cos(2*phi)*(-1 + CosH(2*Omega)) + 
     2      SigmaVCSPol0(2)*CosH(2*Omega) 
     3    + SigmaVCSPol0(5)*BeamHeli*Sin(phi)*SinH(Omega) + 
     4      SigmaVCSPol0(3)*Cos(phi)*SinH(2*Omega))/(2.*qsq)

        BeamHeli=-1
        SigmaVCSm=(SigmaVCSPol0(1) 
     1    - SigmaVCSPol0(4)*Cos(2*phi)*(-1 + CosH(2*Omega)) + 
     2      SigmaVCSPol0(2)*CosH(2*Omega) 
     3    + SigmaVCSPol0(5)*BeamHeli*Sin(phi)*SinH(Omega) + 
     4      SigmaVCSPol0(3)*Cos(phi)*SinH(2*Omega))/(2.*qsq)
      endif
     
      if (TargetPolar.eq.1) then
        BeamHeli=+1
        SigmaVCSp=(SigmaVCSPolX(1)*BeamHeli*CosH(Omega) - 
     1    SigmaVCSPolX(4)*(-1 + CosH(2*Omega))*Sin(2*phi) + 
     2    SigmaVCSPolX(2)*BeamHeli*Cos(phi)*SinH(Omega) + 
     3    SigmaVCSPolX(3)*Sin(phi)*SinH(2*Omega))/(2.*qsq)

        BeamHeli=-1
        SigmaVCSm=(SigmaVCSPolX(1)*BeamHeli*CosH(Omega) - 
     1    SigmaVCSPolX(4)*(-1 + CosH(2*Omega))*Sin(2*phi) + 
     2    SigmaVCSPolX(2)*BeamHeli*Cos(phi)*SinH(Omega) + 
     3    SigmaVCSPolX(3)*Sin(phi)*SinH(2*Omega))/(2.*qsq)
      endif

      if (TargetPolar.eq.2) then
        BeamHeli=+1
        SigmaVCSp=(SigmaVCSPolY(2) - 
     1    SigmaVCSPolY(3)*Cos(2*phi)*(-1 + CosH(2*Omega)) + 
     2    SigmaVCSPolY(3)*CosH(2*Omega) + 
     3    SigmaVCSPolY(1)*BeamHeli*Sin(phi)*SinH(Omega) + 
     4    SigmaVCSPolY(4)*Cos(phi)*SinH(2*Omega))/(2.*qsq)

        BeamHeli=-1
        SigmaVCSm=(SigmaVCSPolY(2) - 
     1    SigmaVCSPolY(3)*Cos(2*phi)*(-1 + CosH(2*Omega)) + 
     2    SigmaVCSPolY(3)*CosH(2*Omega) + 
     3    SigmaVCSPolY(1)*BeamHeli*Sin(phi)*SinH(Omega) + 
     4    SigmaVCSPolY(4)*Cos(phi)*SinH(2*Omega))/(2.*qsq)
      endif
     
      if (TargetPolar.eq.3) then
        BeamHeli=+1
        SigmaVCSp=(SigmaVCSPolZ(1)*BeamHeli*CosH(Omega) - 
     1    SigmaVCSPolZ(4)*(-1 + CosH(2*Omega))*Sin(2*phi) + 
     2    SigmaVCSPolZ(2)*BeamHeli*Cos(phi)*SinH(Omega) + 
     3    SigmaVCSPolZ(3)*Sin(phi)*SinH(2*Omega))/(2.*qsq)

        BeamHeli=-1
        SigmaVCSm=(SigmaVCSPolZ(1)*BeamHeli*CosH(Omega) - 
     1    SigmaVCSPolZ(4)*(-1 + CosH(2*Omega))*Sin(2*phi) + 
     2    SigmaVCSPolZ(2)*BeamHeli*Cos(phi)*SinH(Omega) + 
     3    SigmaVCSPolZ(3)*Sin(phi)*SinH(2*Omega))/(2.*qsq)
      endif

c-- SqrAmplInterf(Q2Input,xBInput,tInput,phi,BeamHeli,TargetPolar,BeamCharge) --*
c Computes the expansion of the Interference cross section (divided by phase    |
c space) in terms of the "expansion cross sections" computed in the function    |
c MakeExactCrossSections (or MakeLeadingCrossSections).                         |
c-------------------------------------------------------------------------------*
  
      if (TargetPolar.eq.0) then
        BeamHeli=+1
        SigmaIp=-((BeamCharge*(SigmaIPol0(1)*CosH(Omega) + 
     1    SigmaIPol0(3)*Cos(2*phi)*(CosH(Omega) - CosH(3*Omega)) + 
     2    SigmaIPol0(2)*CosH(3*Omega) - 
     3    SigmaIPol0(8)*BeamHeli*(-1 + CosH(2*Omega))*Sin(2*phi) + 
     3    SigmaIPol0(7)*BeamHeli*Sin(phi)*SinH(2*Omega) + 
     4    (SigmaIPol0(6)*Cos(3*phi)*(3*SinH(Omega) - SinH(3*Omega)))/3. + 
     5    Cos(phi)*(SigmaIPol0(3)*SinH(Omega) + 
     6    SigmaIPol0(4)*SinH(3*Omega))))/(DDDC*qsq*tt))

        BeamHeli=-1
        SigmaIm=-((BeamCharge*(SigmaIPol0(1)*CosH(Omega) + 
     1    SigmaIPol0(3)*Cos(2*phi)*(CosH(Omega) - CosH(3*Omega)) + 
     2    SigmaIPol0(2)*CosH(3*Omega) - 
     3    SigmaIPol0(8)*BeamHeli*(-1 + CosH(2*Omega))*Sin(2*phi) + 
     3    SigmaIPol0(7)*BeamHeli*Sin(phi)*SinH(2*Omega) + 
     4    (SigmaIPol0(6)*Cos(3*phi)*(3*SinH(Omega) - SinH(3*Omega)))/3. + 
     5    Cos(phi)*(SigmaIPol0(3)*SinH(Omega) + 
     6    SigmaIPol0(4)*SinH(3*Omega))))/(DDDC*qsq*tt))
      endif
     
      if (TargetPolar.eq.1) then
        BeamHeli=+1
        SigmaIp=-((BeamCharge*(-(SigmaIPolX(8)*BeamHeli*Cos(2*phi)*
     1    (-1 + CosH(2*Omega))) + 
     2    BeamHeli*(SigmaIPolX(5) + SigmaIPolX(6)*CosH(2*Omega)) + 
     3    SigmaIPolX(3)*(CosH(Omega) - CosH(3*Omega))*Sin(2*phi) + 
     4    SigmaIPolX(7)*BeamHeli*Cos(phi)*SinH(2*Omega) + 
     5    (SigmaIPolX(4)*Sin(3*phi)*(3*SinH(Omega)-SinH(3*Omega)))/3. + 
     6    Sin(phi)*(SigmaIPolX(1)*SinH(Omega) + 
     7    SigmaIPolX(2)*SinH(3*Omega))))/(DDDC*qsq*tt))

        BeamHeli=-1
        SigmaIm=-((BeamCharge*(-(SigmaIPolX(8)*BeamHeli*Cos(2*phi)*
     1    (-1 + CosH(2*Omega))) + 
     2    BeamHeli*(SigmaIPolX(5) + SigmaIPolX(6)*CosH(2*Omega)) + 
     3    SigmaIPolX(3)*(CosH(Omega) - CosH(3*Omega))*Sin(2*phi) + 
     4    SigmaIPolX(7)*BeamHeli*Cos(phi)*SinH(2*Omega) + 
     5    (SigmaIPolX(4)*Sin(3*phi)*(3*SinH(Omega)-SinH(3*Omega)))/3. + 
     6    Sin(phi)*(SigmaIPolX(1)*SinH(Omega) + 
     7    SigmaIPolX(2)*SinH(3*Omega))))/(DDDC*qsq*tt))
      endif

      if (TargetPolar.eq.2) then
        BeamHeli=+1
        SigmaIp=-((BeamCharge*(SigmaIPolY(1)*CosH(Omega) + 
     1    SigmaIPolY(5)*Cos(2*phi)*(CosH(Omega) - CosH(3*Omega)) + 
     2    SigmaIPolY(2)*CosH(3*Omega) - 
     3    SigmaIPolY(8)*BeamHeli*(-1 + CosH(2*Omega))*Sin(2*phi) + 
     4    SigmaIPolY(7)*BeamHeli*Sin(phi)*SinH(2*Omega) + 
     5    (SigmaIPolY(6)*Cos(3*phi)*(3*SinH(Omega)-SinH(3*Omega)))/3. + 
     6    Cos(phi)*(SigmaIPolY(3)*SinH(Omega) + 
     7    SigmaIPolY(4)*SinH(3*Omega))))/(DDDC*qsq*tt))

        BeamHeli=-1
        SigmaIm=-((BeamCharge*(SigmaIPolY(1)*CosH(Omega) + 
     1    SigmaIPolY(5)*Cos(2*phi)*(CosH(Omega) - CosH(3*Omega)) + 
     2    SigmaIPolY(2)*CosH(3*Omega) - 
     3    SigmaIPolY(8)*BeamHeli*(-1 + CosH(2*Omega))*Sin(2*phi) + 
     4    SigmaIPolY(7)*BeamHeli*Sin(phi)*SinH(2*Omega) + 
     5    (SigmaIPolY(6)*Cos(3*phi)*(3*SinH(Omega)-SinH(3*Omega)))/3. + 
     6    Cos(phi)*(SigmaIPolY(3)*SinH(Omega) + 
     7    SigmaIPolY(4)*SinH(3*Omega))))/(DDDC*qsq*tt))
      endif
     
      if (TargetPolar.eq.3) then
        BeamHeli=+1
        SigmaIp=-((BeamCharge*(-(SigmaIPolZ(8)*BeamHeli*Cos(2*phi)*
     1    (-1 + CosH(2*Omega))) + 
     2    he*(SigmaIPolZ(5) + SigmaIPolZ(6)*CosH(2*Omega)) + 
     3    SigmaIPolZ(3)*(CosH(Omega) - CosH(3*Omega))*Sin(2*phi) + 
     4    SigmaIPolZ(7)*BeamHeli*Cos(phi)*SinH(2*Omega) + 
     5    (SigmaIPolZ(4)*Sin(3*phi)*(3*SinH(Omega) - SinH(3*Omega)))/3. + 
     6    Sin(phi)*(SigmaIPolZ(1)*SinH(Omega) + 
     7    SigmaIPolZ(2)*SinH(3*Omega))))/(DDDC*qsq*tt))

        BeamHeli=-1
        SigmaIm=-((BeamCharge*(-(SigmaIPolZ(8)*BeamHeli*Cos(2*phi)*
     1    (-1 + CosH(2*Omega))) + 
     2    he*(SigmaIPolZ(5) + SigmaIPolZ(6)*CosH(2*Omega)) + 
     3    SigmaIPolZ(3)*(CosH(Omega) - CosH(3*Omega))*Sin(2*phi) + 
     4    SigmaIPolZ(7)*BeamHeli*Cos(phi)*SinH(2*Omega) + 
     5    (SigmaIPolZ(4)*Sin(3*phi)*(3*SinH(Omega) - SinH(3*Omega)))/3. + 
     6    Sin(phi)*(SigmaIPolZ(1)*SinH(Omega) + 
     7    SigmaIPolZ(2)*SinH(3*Omega))))/(DDDC*qsq*tt))
      endif

c-- CrossSectionBH(Q2Input,xBInput,tInput,phi,BeamHeli,TargetPolar,BeamCharge) -*
c Computes the expansion of the Bethe Heitler cross section in terms of the     |
c "expansion cross sections" computed in the function MakeExactCrossSections    | 
c (or MakeLeadingCrossSections).                                                |
c-------------------------------------------------------------------------------*
 
      XSecBHp = SigmaBHp * PhaseSpace
      XSecBHm = SigmaBHm * PhaseSpace
      
c- CrossSectionVCS(Q2Input,xBInput,tInput,phi,BeamHeli,TargetPolar,BeamCharge) -*
c Computes the expansion of the Virtual Compton Scattering cross section in     |
c terms of the "expansion cross sections" computed in the function              |
c MakeExactCrossSections (or MakeLeadingCrossSections).                         |
c-------------------------------------------------------------------------------*
 
      XSecVCSp = SigmaVCSp * PhaseSpace
      XSecVCSm = SigmaVCSm * PhaseSpace
      
c-CrossSectionInterf(Q2Input,xBInput,tInput,phi,BeamHeli,TargetPolar,BeamCharge)*
c Computes the expansion of the Interference cross section in terms of the      |
c "expansion cross sections" computed in the function MakeExactCrossSections    |
c (or MakeLeadingCrossSections).                                                |
c-------------------------------------------------------------------------------*
  
      XSecIp = SigmaIp * PhaseSpace
      XSecIm = SigmaIm * PhaseSpace

c-------------------------------------------------------------------------------*
c the routine returns d4sigma/dQ2dxdtdphi in nb/GeV4
c i.e. there already is a virtual photon flux factor built in (see Brauel Eqn 2.2)

      SigmaTotPlus  = XSecBHp + XSecVCSp + XSecIp
      SigmaTotMoins = XSecBHm + XSecVCSm + XSecIm

      XSecSum=Pi * ( SigmaTotPlus + SigmaTotMoins ) * ConvGeV2nbarn
      XSecDif=Pi * ( SigmaTotPlus - SigmaTotMoins ) * ConvGeV2nbarn

      if (opt.eq.1) then  ! BH cross section only
         sig_tgendvcs=Pi*(XSecBHp+XSecBHm)* ConvGeV2nbarn
      else
         sig_tgendvcs=XSecSum
      endif

c SIMC requires d6sigma/dOmega_p/dE_p/dOmega_e/dE_e [ub/MeV^2/sr^2]
c multiply by Jacobian computed in logbook, pages 107,112,113
c conversion to MeV units is done in physics_dvcs.f

      Jac6=(2./cos(thetacm)) * ((qCM(3)*Ep/Pp)-Ep-qCM(4)) *
     1    ( (Elab**2)*E_prime*(1-cos(thetaEscat) ) / 
     2    (pi*m_p*(Elab-E_prime)) * (1.-Elab/(Elab-E_prime))

      sig_tgvdvcs = sig_tgendvcs*Jac6

      return

